"""
The name list was sourced from UCD and DCU project
 https://www.duchas.ie/en/nom

first names from Northern Ireland Statistics and Research Agency
 https://www.nisra.gov.uk/publications/baby-names-2016
1996 series
"""
from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats = (
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}-{{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}-{{last_name}}',
        '{{prefix_male}} {{first_name_male}} {{last_name}}',
        '{{prefix_female}} {{first_name_female}} {{last_name}}',
        '{{prefix_male}} {{first_name_male}} {{last_name}}',
        '{{prefix_female}} {{first_name_female}} {{last_name}}',
    )
    first_names_male = (
        "Aaron", "Adam", "Adrian", "Aedan", "Aidan", "Aiden", "Alan", "Alastair", "Albert", "Alex", "Alexander",
        "Alistair", "Alister", "Andrew", "Angus", "Anthony", "Antoin", "Anton", "Aodhan", "Arran", "Arron", "Ashley",
        "Bailey", "Bailie", "Barry", "Ben", "Benjamin", "Benn", "Bernard", "Blaine", "Blake", "Brad", "Bradley",
        "Brandon", "Breandan", "Brendan", "Brett", "Brian", "Bryan", "Cahal", "Cahir", "Cailum", "Cal", "Caleb",
        "Callan", "Callum", "Calum", "Calvin", "Cameron", "Caoimhin", "Caolain", "Caolan", "Caomhan", "Carl", "Carter",
        "Cathal", "Charles", "Charlie", "Che", "Chris", "Christian", "Christie", "Christopher", "Christy", "Cianan",
        "Ciaran", "Cillian", "Clark", "Clifford", "Cody", "Colin", "Colm", "Colum", "Conal", "Conall", "Conan",
        "Conchur", "Conn", "Connor", "Conor", "Conrad", "Corey", "Cormac", "Cory", "Craig", "Curtis", "Daire", "Dale",
        "Damian", "Damien", "Daniel", "Danny", "Dara", "Darragh", "Darrell", "Darren", "Darryl", "Daryl", "David",
        "Deaglan", "Dean", "Deane", "Declan", "Dennis", "Dermot", "Desmond", "Diarmuid", "Dillon", "Domhnall",
        "Dominic", "Don", "Donal", "Duncan", "Dylan", "Eamon", "Eamonn", "Edward", "Elliot", "Emmet", "Emmett", "Enda",
        "Eoghan", "Eoin", "Eric", "Ethan", "Euan", "Eugene", "Eunan", "Evan", "Ewan", "Feargal", "Fearghal", "Fergal",
        "Fergus", "Finbar", "Finn", "Fintan", "Fionntan", "Francis", "Frazer", "Gabriel", "Gareth", "Garrett", "Gary",
        "Gavin", "Geoffrey", "George", "Gerald", "Gerard", "Giles", "Glen", "Glenn", "Gordon", "Graeme", "Graham",
        "Grant", "Gregory", "Hamish", "Harry", "Harvey", "Hayden", "Henry", "Hugh", "Iain", "Ian", "Isaac", "Jack",
        "Jackson", "Jacob", "Jaime", "Jake", "James", "Jamie", "Jared", "Jarlath", "Jason", "Jay", "Jeffrey", "Jesse",
        "Joe", "Joel", "John", "Johnathan", "Johnny", "Jon", "Jonathan", "Jonathon", "Jordan", "Jordon", "Joseph",
        "Josh", "Joshua", "Jude", "Justin", "Kane", "Karl", "Kealan", "Keelan", "Keith", "Kelvin", "Kenneth", "Kevin",
        "Kieran", "Killian", "Kirk", "Kristian", "Kristopher", "Kurt", "Kurtis", "Kyle", "Lee", "Leo", "Leon", "Lewis",
        "Liam", "Lloyd", "Logan", "Lorcan", "Louis", "Lucas", "Luke", "Lyndon", "Macauley", "Mairtin", "Malachy",
        "Malcolm", "Manus", "Marc", "Marco", "Marcus", "Mark", "Martin", "Matthew", "Max", "Michael", "Micheal",
        "Mitchel", "Mitchell", "Morgan", "Myles", "Naoise", "Nathan", "Nathaniel", "Neil", "Niall", "Nicholas", "Nigel",
        "Noel", "Odhran", "Oisin", "Oliver", "Omar", "Oran", "Owen", "Padraic", "Padraig", "Patrick", "Paul", "Pauric",
        "Peadar", "Pearce", "Pearse", "Peter", "Philip", "Phillip", "Piaras", "Pierce", "Raymond", "Reece", "Reuben",
        "Rhys", "Rian", "Richard", "Robbie", "Robert", "Robin", "Rohan", "Ronan", "Rory", "Ross", "Rowan", "Roy",
        "Ruairi", "Ruari", "Russell", "Ryan", "Sam", "Samuel", "Saul", "Scot", "Scott", "Seamus", "Sean", "Sean-Paul",
        "Shane", "Shaun", "Shay", "Shea", "Simon", "Stefan", "Stephen", "Steven", "Stewart", "Stuart", "Taylor",
        "Terence", "Thomas", "Tiarnan", "Tiernan", "Timothy", "Tobias", "Toby", "Tom", "Tomas", "Tony", "Travis",
        "Trevor", "Tristan", "Troy", "Tyler", "Tyrone", "Vincent", "Warren", "Wayne", "William", "Zac", "Zach",
        "Zachary", "Zak",
    )

    first_names_female = (
        "Abbi", "Abbie", "Abby", "Abigail", "Adele", "Aideen", "Aileen", "Ailis", "Aimee", "Aine", "Aisling", "Aislinn",
        "Alana", "Alanis", "Alanna", "Alannah", "Alex", "Alexandra", "Alexandria", "Alice", "Alicia", "Alisha",
        "Alison", "Alix", "Amanda", "Amber", "Amelia", "Amie", "Amy", "Amy-Lee", "Amy-Leigh", "Anastasia", "Andrea",
        "Angela", "Anna", "Annalise", "Anne-Marie", "Annie", "Antoinette", "Aoibheann", "Aoibhin", "Aoibhinn", "Aoife",
        "April", "Arianne", "Ashleigh", "Ashlene", "Ashley", "Ashling", "Ashton", "Ayesha", "Bernadette", "Beth",
        "Bethan", "Bethany", "Billie-Jo", "Blanaid", "Brigid", "Brittany", "Brogan", "Bronach", "Bronagh", "Brooke",
        "Brooklyn", "Bryony", "Cailin", "Caitlin", "Caitlyn", "Caitriona", "Caoilfhionn", "Caoimhe", "Cara", "Caragh",
        "Carla", "Carly", "Carmel", "Carol", "Caroline", "Carolyn", "Carrie", "Casey", "Cassandra", "Cassie",
        "Catherine", "Cathy", "Catriona", "Ceara", "Celine", "Chantel", "Chantelle", "Charis", "Charlene", "Charlie",
        "Charlotte", "Chelsea", "Chelsey", "Cherie", "Cherith", "Chloe", "Christina", "Christine", "Ciara", "Ciarrai",
        "Claire", "Clara", "Clare", "Clarissa", "Claudia", "Cliodhna", "Cliona", "Clodagh", "Codie", "Colleen",
        "Collette", "Connie", "Constance", "Cora", "Corinne", "Corrie", "Cortney", "Courteney", "Courtney", "Daire",
        "Dairine", "Dana", "Danielle", "Dara", "Darcy", "Darragh", "Dawn", "Dayna", "Dearbhail", "Dearbhaile",
        "Dearbhla", "Deborah", "Deirbhile", "Demi", "Demi-Lee", "Demi-Leigh", "Denise", "Dervla", "Diane", "Dionne",
        "Donna", "Eadaoin", "Ebony", "Edel", "Eden", "Eileen", "Eilis", "Eilish", "Eimear", "Eimer", "Eimhear",
        "Elaine", "Eleanor", "Elise", "Elisha", "Elizabeth", "Ella", "Ellen", "Ellie", "Eloise", "Emer", "Emilie",
        "Emily", "Emma", "Emma-Louise", "Enya", "Erica", "Erika", "Erin", "Eryn", "Esther", "Eva", "Eve", "Evelyn",
        "Evie", "Fainche", "Faith", "Faye", "Fiona", "Fionnuala", "Frances", "Francesca", "Freya", "Gabrielle", "Gemma",
        "Georgia", "Georgina", "Geraldine", "Gillian", "Gina", "Grace", "Grainne", "Haley", "Hannah", "Harriet",
        "Hayleigh", "Hayley", "Heather", "Heidi", "Helen", "Helena", "Hollie", "Holly", "India", "Iona", "Jacqueline",
        "Jade", "Jamie", "Jamie-Lee", "Jamie-Leigh", "Jana", "Jane", "Janet", "Janice", "Janine", "Jasmin", "Jasmine",
        "Jayde", "Jayne", "Jemma", "Jena", "Jenna", "Jenni", "Jennifer", "Jenny", "Jessica", "Jill", "Joanna", "Joanne",
        "Jodie", "Jody", "Johanna", "Jolene", "Jordan", "Josephine", "Joy", "Judith", "Julia", "Julie", "Julie-Anne",
        "Justine", "Kaitlin", "Kaitlyn", "Kara", "Karen", "Karla", "Karley", "Kate", "Katelyn", "Katharine",
        "Katherine", "Kathleen", "Kathryn", "Kathy", "Katie", "Katie-Louise", "Katrina", "Katy", "Kayleigh", "Keely",
        "Keeva", "Kellie", "Kelly", "Kelly-Anne", "Kelly-Marie", "Kelsey", "Keri", "Kerri", "Kerrie", "Kerry", "Kiera",
        "Kimberly", "Kira", "Kirby", "Kirsten", "Kirstie", "Kirstin", "Kirsty", "Kori", "Kristin", "Kristina", "Lana",
        "Laoise", "Lara", "Laura", "Lauren", "Laurie", "Leah", "Leanne", "Leigh", "Leona", "Leonie", "Lesley",
        "Lindsay", "Lisa", "Lisa-Marie", "Lois", "Lorna", "Louise", "Lucia", "Lucinda", "Lucy", "Lydia", "Lynda",
        "Lyndsay", "Lyndsey", "Lynsey", "Madison", "Maeve", "Mairead", "Margaret", "Maria", "Marie", "Marie-Claire",
        "Martha", "Martina", "Mary", "Maura", "Maureen", "Meabh", "Meaghan", "Meg", "Megan", "Meghan", "Meibh",
        "Melanie", "Melissa", "Mia", "Michaela", "Micheala", "Michelle", "Miriam", "Mollie", "Molly", "Morgan", "Nadia",
        "Nadine", "Naoimh", "Naoise", "Naomh", "Naomi", "Natalie", "Natasha", "Niamh", "Nichola", "Nichole", "Nicola",
        "Nicole", "Nikita", "Nikki", "Nina", "Nora", "Nuala", "Olivia", "Oonagh", "Orfhlaith", "Orla", "Orlagh",
        "Orlaigh", "Orlaith", "Padraigin", "Paige", "Patrice", "Patricia", "Paula", "Phoebe", "Polly", "Rachael",
        "Rachel", "Rachelle", "Rebecca", "Rebekah", "Regan", "Rhian", "Rhianna", "Rhianne", "Rhiannon", "Roberta",
        "Robyn", "Roise", "Roisin", "Rose", "Roseanna", "Rosemary", "Rosie", "Ruth", "Sabrina", "Sacha", "Samantha",
        "Sandra", "Saoirse", "Sara", "Sarah", "Sarah-Jane", "Sarah-Louise", "Sasha", "Saskia", "Savannah", "Seana",
        "Seanan", "Seaneen", "Seanna", "Selina", "Seona", "Serena", "Shania", "Shanice", "Shanna", "Shannan", "Shannen",
        "Shannon", "Sharon", "Shauna", "Shauneen", "Shelby", "Shelley", "Sheree", "Shona", "Sian", "Simone", "Sinead",
        "Siobhan", "Siofra", "Sophia", "Sophie", "Sophie-Louise", "Sorcha", "Stacey", "Stephanie", "Susan", "Susanna",
        "Susannah", "Suzanne", "Tamara", "Tammy", "Tanya", "Tara", "Taylor", "Teresa", "Terri", "Tess", "Tessa",
        "Theresa", "Therese", "Tia", "Tiarna", "Tiegan", "Tiffany", "Toni", "Tonicha", "Tori", "Tory", "Tracey",
        "Tyler", "Una", "Ursula", "Vanessa", "Victoria", "Whitney", "Yasmin", "Yasmine", "Zara", "Zoe",
    )

    first_names = first_names_male + first_names_female

    last_names = (
        "Achison", "Adams", "Agnew", "Ahearn", "Ahearne", "Ahern", "Aherne", "Ainsboro", "Allen", "Allis", "Anderson",
        "Andrews", "Angus", "Annsboro", "Ansboro", "Arthurs", "Ashe", "Ashman", "Atchison", "Atkins", "Atkinson",
        "Aylward", "Baker", "Baldwin", "Bale", "Bandeville", "Banks", "Bann", "Bannon", "Banville", "Barnes", "Barnett",
        "Barneville", "Barrett", "Barrnette", "Barron", "Barry", "Bartley", "Bates", "Baxter", "Beakey", "Beal",
        "Beale", "Beasty", "Beattie", "Beatty", "Beggan", "Beggs", "Begley", "Behan", "Beirn", "Beirne", "Bell",
        "Belton", "Bennet", "Bennett", "Beresford", "Bergin", "Bermingham", "Berminghim", "Bernard", "Berney", "Bernie",
        "Berry", "Biesty", "Bird", "Birmingham", "Bishop", "Black", "Blake", "Blanch", "Blanche", "Bodkin", "Bogan",
        "Bohan", "Boland", "Boles", "Bolger", "Bonar", "Boner", "Bones", "Bonner", "Boreland", "Borland", "Bourke",
        "Bowe", "Bowen", "Bowler", "Bowles", "Boyce", "Boylan", "Boyle", "Boyse", "Bradden", "Bradley", "Brady",
        "Branaola", "Brannelly", "Brassil", "Bray", "Bree", "Breen", "Breheny", "Brennan", "Breslin", "Bresnehan",
        "Brett", "Brick", "Bridge", "Bridson", "Brien", "Briody", "Brislane", "Broderick", "Brody", "Brogan", "Brophy",
        "Brosnan", "Brown", "Browne", "Broy", "Bruen", "Bruton", "Bryan", "Bryson", "Buckley", "Burchill", "Burke",
        "Burns", "Burton", "Butler", "Buttimer", "Buttimore", "Byrne", "Byrnes", "Cadden", "Caddow", "Cadogan",
        "Cafferkey", "Cafferky", "Cafferty", "Caffrey", "Cagney", "Cahalane", "Cahill", "Cahillane", "Cahir", "Caine",
        "Cairn", "Cairns", "Caldwell", "Callaghan", "Callan", "Callanan", "Calligan", "Callinan", "Cally", "Calvey",
        "Campbell", "Canavan", "Cannan", "Canniffe", "Canning", "Cannon", "Canny", "Cantwell", "Caplis", "Capples",
        "Capua", "Carbery", "Carey", "Carleton", "Carley", "Carlin", "Carmody", "Carney", "Carolan", "Carr",
        "Carragher", "Carrig", "Carrigan", "Carrigy", "Carroll", "Carry", "Carter", "Carthy", "Carton", "Carty",
        "Carville", "Casey", "Cashen", "Cashman", "Cassen", "Casserley", "Casserly", "Cassidy", "Cassin", "Cattigan",
        "Cauley", "Caulfield", "Cavanagh", "Cawley", "Charles", "Christopher", "Clafferty", "Claffey", "Clair",
        "Clancy", "Clare", "Clarke", "Classon", "Clavin", "Clear", "Cleary", "Clements", "Clenaghan", "Clerkin",
        "Clery", "Clifford", "Clinten", "Clinton", "Clogherty", "Cloherty", "Clohessey", "Clohessy", "Cloney",
        "Cloonan", "Cloone", "Clooney", "Clune", "Coady", "Coakley", "Cody", "Coen", "Coffey", "Cogan", "Cogley",
        "Cohalan", "Cohen", "Coholan", "Cole", "Coleman", "Colfer", "Colgan", "Colhoun", "Coll", "Collen", "Colleneler",
        "Colleran", "Colley", "Collier", "Colligan", "Collinder", "Collins", "Colly", "Colreavy", "Colum", "Comber",
        "Combre", "Comer", "Comerford", "Comisky", "Commins", "Comyn", "Conaty", "Conboy", "Concannon", "Condon",
        "Condren", "Condron", "Conefrey", "Conlan", "Conlon", "Conmee", "Conmy", "Connachton", "Connaghy",
        "Connaughton", "Conneeley", "Conneely", "Connell", "Connellan", "Connelly", "Connery", "Connole", "Connolly",
        "Connor", "Connors", "Conole", "Conree", "Conroy", "Conry", "Considine", "Convey", "Conway", "Conwell",
        "Coogan", "Cook", "Cooke", "Coolahan", "Coonan", "Cooney", "Corbett", "Corcoran", "Corduff", "Corish",
        "Corkery", "Corless", "Corley", "Cormack", "Cormican", "Cormick", "Cormy", "Corr", "Corridan", "Corrigan",
        "Corry", "Cosgrave", "Cosgrove", "Costello", "Costelloe", "Costigan", "Cotter", "Coughlan", "Counihan",
        "Courcey", "Cournane", "Courtenay", "Courtney", "Cousins", "Cowan", "Cowely", "Cowen", "Cowley", "Cox", "Coyle",
        "Coyne", "Crahan", "Craig", "Craine", "Crampsey", "Crampsie", "Crane", "Crangle", "Cranley", "Cranly", "Craven",
        "Crawley", "Crean", "Creed", "Creedon", "Cregan", "Crehan", "Cremin", "Cribbons", "Crilly", "Crimmins",
        "Crinion", "Croal", "Crohan", "Crolly", "Cronelly", "Cronin", "Cronly", "Crosbie", "Crosby", "Cross", "Crossan",
        "Crota", "Crotty", "Crowe", "Crowley", "Crudden", "Cruise", "Cryan", "Cuddihy", "Cuffe", "Culhane", "Cullen",
        "Culligan", "Cullinan", "Cullinane", "Culloty", "Cully", "Cumiskey", "Cumisky", "Cummins", "Cummiskey",
        "Cummisky", "Cunnane", "Cunneen", "Cunningham", "Cunny", "Curley", "Curnane", "Curneen", "Curnyn", "Curran",
        "Currie", "Curry", "Curtin", "Curtis", "Cusack", "D'Arcy", "Daiken", "Dalton", "Daly", "Danaher", "Dane",
        "Daniel", "Daniels", "Darcy", "Dargan", "Darmody", "Dasey", "Davenport", "Davern", "Davey", "Davin", "Davis",
        "Davitt", "Davoren", "Davy", "Daw", "Dawson", "Day", "Deacon", "Deacy", "Deady", "Dean", "Deane", "Dease",
        "Deasy", "Dee", "Deegadan", "Deegan", "Deehan", "Deeley", "Deely", "Deeney", "Deeny", "Deere", "Deery",
        "Deigan", "Deignan", "Delahunty", "Delaney", "Delap", "Delargy", "Deloughrey", "Deloughry", "Dempsey",
        "Denihan", "Denis", "Denison", "Dennehy", "Denning", "Denny", "Dermody", "Dermott", "Derrig", "Desmond",
        "Devally", "Devane", "Devaney", "Devanney", "Devenney", "Dever", "Devereaux Deaueroux", "Devereux", "Devery",
        "Devilly", "Devin", "Devine", "Devitt", "Devlin", "Devoy", "Dickey", "Dickie", "Dickson", "Diffin", "Diffley",
        "Diggin", "Diggins", "Dignan", "Dillane", "Dillon", "Dinan", "Dineen", "Dinneen", "Dirrane", "Diskin",
        "Divenney", "Diver", "Divine", "Diviney", "Dixon", "Dobbin", "Dobbins", "Dogherty", "Doherty", "Dolan",
        "Donagher", "Donaldson", "Donegan", "Donlon", "Donnan", "Donnell", "Donnellan", "Donnelly", "Donoghue",
        "Donohoe", "Donohue", "Donovan", "Doody", "Dooey", "Doogan", "Doohan", "Doolan", "Dooley", "Doorty", "Doran",
        "Dordan", "Dore", "Dorgan", "Dornan", "Dorrian", "Doudigan", "Dowd", "Dower", "Dowey", "Dowley", "Dowling",
        "Downes", "Downey", "Downing", "Doyle", "Drennan", "Drian", "Driscoll", "Drohan", "Droney", "Drum", "Drumm",
        "Drummond", "Drummy", "Duane", "Duff", "Duffin", "Duffy", "Duggan", "Duhig", "Duhy", "Duignan", "Dulohery",
        "Duncan", "Dunford", "Dungan", "Dunleavey", "Dunleavy", "Dunne", "Dunning", "Dunny", "Dunphy", "Dunworth",
        "Durkan", "Durkin", "Durnan", "Durnin", "Durning", "Durrihy", "Dwane", "Dwyer", "Dyer", "Earl", "Earle",
        "Early", "Egan", "Eivers", "Elliot", "Elliott", "Ellis", "Elwood", "English", "Ennis", "Enright", "Ervin",
        "Ervine", "Eustace", "Evans", "Evoy", "Fadden", "Fadian", "Fagan", "Faherty", "Fahey", "Fahy", "Fair", "Fall",
        "Fallon", "Falvey", "Fannin", "Fanning", "Fannon", "Farell", "Farnan", "Farnon", "Farragher", "Farrell",
        "Farrelly", "Farren", "Farrissey", "Farrissy", "Farry", "Faulkner", "Faull", "Fay", "Fealy", "Fearon", "Fee",
        "Feehan", "Feeley", "Feely", "Feeney", "Feeny", "Fegan", "Fehan", "Fehilly", "Feighery", "Felban", "Fenelon",
        "Fenighty", "Fenlon", "Fennell", "Fennelly", "Fennessey", "Fenning", "Fenton", "Fergus", "Ferguson", "Ferris",
        "Ferriter", "Ferry", "Field", "Fielding", "Filban", "Filbin", "Finan", "Finegan", "Finlay", "Finn", "Finnegan",
        "Finneran", "Finnerty", "Finnucane", "Finucane", "Fisher", "Fitzgerald", "Fitzgibbon", "Fitzgibbons",
        "Fitzmartin", "Fitzmaurice", "Fitzpatrick", "Fitzsimmons", "Fitzsimons", "Flaherty", "Flahive", "Flanagan",
        "Flannagan", "Flannelly", "Flannery", "Flatley", "Flavin", "Fleming", "Flinn", "Flood", "Flynn", "Fogarty",
        "Folan", "Foley", "Foody", "Foran", "Forbes", "Ford", "Forde", "Forkin", "Fox", "Foy", "Foyle", "Fraher",
        "Frances", "Francis", "Franklin", "Frawley", "Freaney", "Freeley", "Freely", "Freeney", "Freil", "Fresh",
        "Friel", "Furey", "Fyfe", "Gaffney", "Gahan", "Gaine", "Gainey", "Gallagher", "Gallaher", "Gallen", "Galligan",
        "Gallivan", "Gallogly", "Galvin", "Ganley", "Ganly", "Gannon", "Garavan", "Garde", "Garety", "Gargan",
        "Garland", "Garraghy", "Garrahy", "Garrihy", "Garry", "Gartlan", "Gartland", "Garvey", "Garvin", "Gately",
        "Gaughan", "Gavaghan", "Gavican", "Gavigan", "Gavin", "Gay", "Gaynard", "Gaynor", "Geany", "Gearty", "Geary",
        "Geherty", "Geoghegan", "Geraghty", "Gerarghty", "Gibbon", "Gibbons", "Giblin", "Gibney", "Gibson", "Gilbane",
        "Gilbride", "Gildea", "Gilduff", "Giles", "Gilgunn", "Gilhooly", "Gill", "Gillan", "Gillen", "Gillespie",
        "Gillic", "Gillick", "Gilligan", "Gilliland", "Gillis", "Gillooly", "Gilmartin", "Gilmore", "Gilroy",
        "Gilsenan", "Ginevan", "Ging", "Ginnitty", "Ginnity", "Ginty", "Girvan", "Givern", "Glavin", "Glazier",
        "Gleasure", "Gleeson", "Glennon", "Gloster", "Glynn", "Godfrey", "Goff", "Gogan", "Gogarty", "Goggin", "Golden",
        "Golding", "Goldrick", "Gollan", "Goodwin", "Gorevan", "Gorey", "Gorham", "Gorman", "Gough", "Goulden",
        "Goulding", "Grace", "Grady", "Graham", "Grahams", "Grattan", "Gray", "Grealish", "Greally", "Greaney",
        "Greehy", "Greelish", "Greely", "Green", "Greene", "Grennan", "Grey", "Griffen", "Griffin", "Griffith",
        "Griffiths", "Groarke", "Grogan", "Groogan", "Growney", "Gubain", "Gubben", "Guerin", "Guihan", "Guilfoyle",
        "Guinan", "Guinane", "Guinevan", "Guiney", "Guinnane", "Guinness", "Guiry", "Gunn", "Gunning", "Gwynn",
        "Hackett", "Hagan", "Haggerty", "Hahessy", "Haire", "Hallahan", "Hallanan", "Halley", "Hallinan", "Hallissey",
        "Halloran", "Halpen", "Halpin", "Hamilton", "Hanafin", "Hanbury", "Hankard", "Hanley", "Hanlon", "Hanly",
        "Hanna", "Hannah", "Hanncard", "Hannigan", "Hannon", "Hanrahan", "Hanratty", "Hara", "Harahoe", "Haran",
        "Hardiman", "Hardy", "Hare", "Haren", "Hargadon", "Hargan", "Harkin", "Harkins", "Harley", "Harmon", "Harnett",
        "Harrihy", "Harrington", "Harris", "Harrison", "Harry", "Harte", "Hartigan", "Hartnett", "Harty", "Hassett",
        "Hastey", "Hastie", "Hastings", "Hasty", "Hatton", "Haugh", "Haughey", "Haverty", "Hawe", "Hawthorn", "Hayden",
        "Hayes", "Heaffy", "Healy", "Heaney", "Heaphy", "Hearn", "Hearne", "Hearty", "Heavey", "Heckett", "Hedderman",
        "Hedigan", "Heelan", "Heenan", "Heeney", "Heffernan", "Hefferon", "Heffron", "Hegarty", "Heggarty", "Hehir",
        "Helen", "Helery", "Hely", "Hempenstall", "Hendry", "Henebry", "Heneghan", "Henery", "Heney", "Hennebry",
        "Hennelley", "Hennelly", "Hennessey", "Hennessy", "Hennigan", "Henry", "Hepenstall", "Heraghty", "Heraty",
        "Herbert", "Hereward", "Herity", "Herlihy", "Hernon", "Heron", "Heskin", "Heslin", "Hession", "Hever", "Hewson",
        "Hickey", "Higgins", "Hilary", "Hillen", "Hillery", "Hilliard", "Hinney", "Hishon", "Histon", "Hoare", "Hoban",
        "Hodnett", "Hoey", "Hogan", "Holden", "Holland", "Hollins", "Hollywood", "Holmes", "Holohan", "Honan",
        "Hopkins", "Horan", "Hore", "Horgan", "Hosae", "Hosey", "Hoskins", "Hough", "Houlihan", "Hourican", "Hourigan",
        "Hourihane", "Howard", "Howe", "Howley", "Hughes", "Humphreys", "Hunt", "Hunter", "Hurd", "Hurley", "Hussey",
        "Hutchinson", "Hutchison", "Hutton", "Hyde", "Hyland", "Hyman", "Hynes", "Iago", "Igoe", "Inglis", "Ingoldsby",
        "Irvine", "Irwin", "Ivers", "Ivory", "Jackman", "Jackson", "Jameson", "Jennings", "Jiles", "Johnson",
        "Johnston", "Johnstone", "Jones", "Jordan", "Joyce", "Judge", "Kane", "Kangley", "Kavanagh", "Keady", "Kealey",
        "Keally", "Kealty", "Kealy", "Keane", "Keaney", "Keany", "Keapock", "Kearney", "Kearns", "Keary", "Keating",
        "Keaveney", "Keaveny", "Keeffe", "Keegan", "Keehan", "Keelan", "Keeley", "Keely", "Keenaghan", "Keenahan",
        "Keenan", "Keeney", "Keery", "Keevers", "Kehoe", "Keightley", "Kelleher", "Keller", "Kelly", "Kelvey", "Kenlan",
        "Kenlon", "Kenna", "Kenneally", "Kennedy", "Kennellan", "Kennelly", "Kenny", "Keogan", "Keogh", "Keoghan",
        "Keoghane", "Keohan", "Keohane", "Keown", "Kerin", "Kerins", "Kerley", "Kerlin", "Kermody", "Kernan", "Kerney",
        "Kerr", "Kerrigan", "Kerrisk", "Kerville", "Kerwick", "Kevane", "Keville", "Keyes", "Kidney", "Kiely", "Kieran",
        "Kierane", "Kierans", "Kiernan", "Kilawee", "Kilbane", "Kilbride", "Kilcoyne", "Kilday", "Kildea", "Kilduff",
        "Kilfoyle", "Kilgallen", "Kilgallon", "Kilhooly", "Kilkenny", "Killeen", "Killilea", "Killooly", "Killoran",
        "Killoughry", "Kilmartin", "Kilmore", "Kilroe", "Kilroy", "Kinaghan", "Kinahan", "King", "Kingston", "Kiniry",
        "Kinlan", "Kinlen", "Kinnane", "Kinnear", "Kinnegan", "Kinner", "Kinnerk", "Kinney", "Kinnon", "Kinny",
        "Kinsella", "Kirby", "Kirke", "Kirwan", "Kissane", "Kitson", "Kneafsey", "Knight", "Kyne", "Lacey", "Lacy",
        "Lafferty", "Laffey", "Lahey", "Lahiffe", "Lahy", "Laing", "Lally", "Lalor", "Lambe", "Lamont", "Landa",
        "Lande", "Landers", "Landy", "Lane", "Lang", "Langan", "Lanigan", "Lappin", "Lardner", "Largan", "Largey",
        "Larkin", "Lavan", "Lavell", "Lavelle", "Laverty", "Lavery", "Lavin", "Lawless", "Lawlor", "Leacy", "Leahy",
        "Leary", "Leavey", "Leddin", "Leddon", "Leddy", "Ledwich", "Ledwith", "Lee", "Leech", "Leen", "Leeney",
        "Lehane", "Leland", "Lenaghan", "Leneghan", "Lenehan", "Lenihan", "Lennane", "Lennon", "Leonard", "Lester",
        "Levan", "Leyden", "Leydon", "Liddane", "Liddy", "Lillis", "Lincoln", "Lindsay", "Linehan", "Linnane", "Linny",
        "Linskey", "Liston", "Little", "Loftus", "Logan", "Loghan", "Logue", "London", "Lonergan", "Long", "Longan",
        "Looney", "Lord", "Lordan", "Loughlin", "Loughnane", "Loughran", "Loughrey", "Loughry", "Lovett", "Lowe",
        "Lowney", "Lowry", "Lucey", "Lucid", "Lucitt", "Luddy", "Lundon", "Lunham", "Lunney", "Lunny", "Lyden", "Lydon",
        "Lynch", "Lynchechaun", "Lynchehaun", "Lyne", "Lyng", "Lynn", "Lynskey", "Lyons", "Lysaght", "Mac Breen",
        "MacAdoo", "MacAleavy", "MacAllen", "MacAloon", "MacAnally", "MacArt", "MacArthur", "MacBreen", "MacBride",
        "MacCaffrey", "MacCann", "MacCartan", "MacCarthy", "MacCarville", "MacClenaghan", "MacCole", "MacComisky",
        "MacConachy", "MacConnaghy", "MacCool", "MacCormack", "MacCurtin", "MacDermott", "MacDevitt", "MacDonagh",
        "MacDonald", "MacDonnell", "MacDougall", "MacDowell", "MacDwyer", "MacDyer", "MacEgan", "MacElgunn", "MacEver",
        "MacEvoy", "MacFadden", "MacFall", "MacFaull", "MacGee", "MacGeehan", "MacGill", "MacGilligan", "MacGing",
        "MacGinley", "MacGinnitty", "MacGinnity", "MacGinty", "MacGloin", "MacGlynn", "MacGovern", "MacGreal",
        "MacGroarty", "MacGuinness", "MacGurk", "MacHale", "MacHenry", "MacHugh", "MacInerney", "MacInnes", "MacKenna",
        "MacKeown", "MacKevitt", "MacLysaght", "MacMahon", "MacMonagle", "MacMorrow", "MacMullan", "MacMullen",
        "MacNabb", "MacNaboe", "MacNaboola", "MacNally", "MacNamara", "MacNamee", "MacNeela", "MacNeill", "MacNelis",
        "MacNulty", "MacPhilbin", "MacShea", "MacSweeney", "MacTiernan", "MacVeagh", "MacVeigh", "MacWilliams",
        "Macauley", "Macken", "Mackesey", "Mackey", "Mackle", "Maclean", "Macmillan", "Macrea", "Madden", "Maddock",
        "Maddy", "Madigan", "Magan", "Magann", "Magauran", "Magee", "Mageean", "Magennis", "Magennity", "Magill",
        "Maginn", "Magrath", "Maguire", "Mahedy", "Maher", "Mahon", "Mahoney", "Mahony", "Malley", "Mallon", "Malone",
        "Maloney", "Malowney", "Manahan", "Mangan", "Manley", "Mann", "Manning", "Mannion", "Mannix", "Mansell",
        "Mansfield", "Mara", "Markey", "Markham", "Marley", "Marnan", "Marren", "Marrinan", "Marron", "Marry", "Martin",
        "Martyn", "Masterson", "Matthews", "Maughan", "Maxwell", "May", "Maye", "McAdams", "McAleavy", "McAleenan",
        "McAleer", "McAlinney", "McAlister", "McAloon", "McAlunny", "McAnally", "McAndrew", "McAnulty", "McArdle",
        "McAreavey", "McAtee", "McAteer", "McAuley", "McAuliffe", "McAveigh", "McBreen", "McBride", "McBrien", "McCabe",
        "McCadam", "McCadden", "McCafferky", "McCafferty", "McCaffrey", "McCaffry", "McCahill", "McCall", "McCallion",
        "McCann", "McCardle", "McCarney", "McCarra", "McCarron", "McCartan", "McCarte", "McCarthy", "McCarville",
        "McCaughan", "McCaughey", "McCaul", "McCauley", "McCausland", "McCay", "McClean", "McClelland", "McCloskey",
        "McCluskey", "McColgan", "McColl", "McCollam", "McComiskey", "McConaghey", "McConaghy", "McConnell", "McConnon",
        "McCooey", "McCool", "McCorkill", "McCorley", "McCormick", "McCorry", "McCourt", "McCoy", "McCracken",
        "McCrann", "McCrea", "McCready", "McCreanor", "McCrory", "McCrossan", "McCrudden", "McCullagh", "McCullough",
        "McCumiskey", "McCumisky", "McCurdy", "McCurley", "McCurtin", "McCusker", "McDade", "McDaeid", "McDaid",
        "McDermod", "McDermott", "McDevitt", "McDonagh", "McDonald", "McDougald", "McDowell", "McDunphy", "McDwyer",
        "McDyer", "McElduff", "McElgunn", "McElhattin", "McEllistrim", "McElnay", "McElnea", "McElroe", "McElroy",
        "McElwaine", "McElwee", "McEnaney", "McEneaney", "McEnry", "McEntaggart", "McEntee", "McEvaddy", "McEvilly",
        "McEvoy", "McFadden", "McFall", "McFarland", "McFaull", "McGahey", "McGalligly", "McGann", "McGarraghy",
        "McGarrigle", "McGarry", "McGarvey", "McGauran", "McGaw", "McGeady", "McGee", "McGeehan", "McGeoghegan",
        "McGeown", "McGerr", "McGettigan", "McGettrick", "McGill", "McGillicuddy", "McGilligan", "McGilly", "McGilroy",
        "McGinley", "McGinnitty", "McGinty", "McGirl", "McGirr", "McGivern", "McGlinchey", "McGlinchy", "McGloin",
        "McGlynn", "McGoff", "McGoldrick", "McGonagle", "McGough", "McGourty", "McGovern", "McGowan", "McGowern",
        "McGrane", "McGrath", "McGreal", "McGrenehan", "McGroarty", "McGrory", "McGruddie", "McGruddy", "McGuigan",
        "McGuill", "McGuinn", "McGuinness", "McGuire", "McGuirk", "McGuirl", "McGurk", "McHale", "McHarry", "McHenry",
        "McHugh", "McIldownie", "McIlroe", "McIlroy", "McIlwee", "McIneely", "McInerney", "McInnes", "McIntyre",
        "McIvor", "McKaigue", "McKay", "McKee", "McKeegan", "McKeever", "McKelvey", "McKendry", "McKeniry", "McKenna",
        "McKenny", "McKeogh", "McKeon", "McKeown", "McKernon", "McKevitt", "McKie", "McKiernan", "McKillop", "McKing",
        "McKinley", "McKinney", "McKinnon", "McKnight", "McLaughlin", "McLaverty", "McLean", "McLeer", "McLeese",
        "McLeigh", "McLeod", "McLoon", "McLoone", "McLoughlin", "McMacken", "McMahon", "McManus", "McMaster",
        "McMenamin", "McMonagle", "McMorrow", "McMullen", "McMurrough", "McNaboe", "McNally", "McNamara", "McNamee",
        "McNaughton", "McNea", "McNealy", "McNee", "McNeely", "McNeill", "McNelis", "McNevin", "McNicholas",
        "McNicholl", "McNill", "McNulty", "McPartland", "McPartlin", "McPartlon", "McPherson", "McPhilbin",
        "McPhillips", "McPolin", "McQuade", "McQuaid", "McQueen", "McQuilkan", "McQuillan", "McQuillen", "McQuin",
        "McQuinn", "McRann", "McReady", "McRoarty", "McRory", "McShane", "McSharry", "McSheehy", "McTeague", "McTernan",
        "McTiernan", "McTigue", "McVeagh", "McVeigh", "McVicker", "McVitty", "McWalter", "Meaghan", "Meagher", "Meaney",
        "Meany", "Meara", "Mee", "Meehan", "Meenaghan", "Meenan", "Megaw", "Mehigan", "Melady", "Meldon", "Melia",
        "Melican", "Mellet", "Mellon", "Melody", "Melville", "Melvin", "Menton", "Mernagh", "Merrigan", "Merry",
        "Mescall", "Meskill", "Miley", "Millar", "Millea", "Miller", "Millet", "Millican", "Milligan", "Milmo", "Milne",
        "Milroy", "Minihan", "Minihane", "Minogue", "Miscell", "Miskell", "Mitchell", "Moan", "Moffatt", "Moffit",
        "Mohan", "Moher", "Molloy", "Moloney", "Molyneux", "Monaghan", "Monagle", "Monahan", "Mongan", "Monk", "Monks",
        "Monroe", "Montague", "Montgomery", "Moody", "Moone", "Mooney", "Moore", "Morahan", "Moran", "Morgan",
        "Moriarty", "Morley", "Mornane", "Moroney", "Morrin", "Morris", "Morrison", "Morrissey", "Morrow", "Mountain",
        "Moy", "Moylan", "Moynihan", "Mulcahy", "Mulcair", "Muldoon", "Muldowney", "Mulgrave", "Mulgrew", "Mulhare",
        "Mulhern", "Mulkerrin", "Mullaghan", "Mullaly", "Mullan", "Mullane", "Mullaney", "Mullany", "Mullarkey",
        "Mullen", "Mullery", "Mulligan", "Mullin", "Mullins", "Mullooly", "Mullooney", "Mulloughney", "Mulloy",
        "Mulqueen", "Mulqueeny", "Mulrain", "Mulrooney", "Mulroy", "Mulry", "Mulryan", "Mulvany", "Mulvenna", "Mulvey",
        "Mulvihill", "Mulvin", "Mulvy", "Munnelly", "Munroe", "Murae", "Murnane", "Murnin", "Murphy", "Murray",
        "Murrihy", "Murtagh", "Myers", "Myles", "Nagle", "Nallon", "Nally", "Nalty", "Nangle", "Nary", "Nash",
        "Naughton", "Nea", "Nealon", "Neary", "Nee", "Needham", "Neehan", "Neelan", "Neelin", "Neenan", "Neilan",
        "Neilian", "Neill", "Neligan", "Nelis", "Nelson", "Nestor", "Neville", "Nevin", "Neylon", "Nicholas",
        "Nicholls", "Nicholson", "Niland", "Nixon", "Nolan", "Nolty", "Noonan", "Noone", "Norris", "Norry", "Norton",
        "Nugent", "Nulty", "Nunne", "Nyhan", "O'Beirn", "O'Beirne", "O'Boyle", "O'Brassil", "O'Brazil", "O'Brennan",
        "O'Brien", "O'Brown", "O'Bryan", "O'Bryen", "O'Byrne", "O'Cadden", "O'Cafferky", "O'Callaghan", "O'Carolan",
        "O'Carroll", "O'Casey", "O'Cassidy", "O'Cleary", "O'Clery", "O'Connell", "O'Connor", "O'Crohan", "O'Crowley",
        "O'Curry", "O'Daly", "O'Dea", "O'Devanney", "O'Devenny", "O'Doherty", "O'Donnell", "O'Donoghue", "O'Donohoe",
        "O'Donovan", "O'Dowd", "O'Driscoll", "O'Duffy", "O'Dwyer", "O'Farrell", "O'Farrelly", "O'Flaherty", "O'Flynn",
        "O'Freil", "O'Friel", "O'Gallagher", "O'Gara", "O'Goldrick", "O'Gorman", "O'Gowan", "O'Grady", "O'Growney",
        "O'Hagan", "O'Haire", "O'Halloran", "O'Hanlon", "O'Hanrahan", "O'Hara", "O'Hare", "O'Haughey", "O'Hea",
        "O'Hegarty", "O'Hehir", "O'Herlihy", "O'Hickey", "O'Higgins", "O'Hora", "O'Houlihan", "O'Hurley", "O'Hussey",
        "O'Kane", "O'Kearney", "O'Keefe", "O'Keeffe", "O'Kelly", "O'Kennedy", "O'Kieve", "O'Leary", "O'Loan",
        "O'Looney", "O'Loughlin", "O'Loughlinn", "O'Mahoney", "O'Mahony", "O'Malley", "O'Mara", "O'Meara", "O'Mooney",
        "O'Moore", "O'Mullan", "O'Murnaghan", "O'Neill", "O'Nolan", "O'Rafferty", "O'Rahilly", "O'Reardon", "O'Regan",
        "O'Reilly", "O'Riordan", "O'Rooney", "O'Rourke", "O'Ruane", "O'Ryan", "O'Scannell", "O'Shannon", "O'Sharkey",
        "O'Shaughnessy", "O'Shea", "O'Sheehan", "O'Sheil", "O'Shiel", "O'Sullivan", "O'Sweeney", "O'Tierney",
        "O'Togher", "O'Toole", "Ormsby", "Owens", "Padden", "Parker", "Parsons", "Paten", "Patterson", "Patton", "Paul",
        "Pender", "Perkins", "Perri", "Perry", "Peyton", "Phayre", "Phelan", "Philban", "Philbin", "Phillips",
        "Piggott", "Pigott", "Pinder", "Plover", "Poland", "Powell", "Power", "Prendergast", "Prial", "Price",
        "Pringle", "Pryal", "Purcell", "Quaide", "Qualter", "Queally", "Queenane", "Quigley", "Quigney", "Quill",
        "Quillinan", "Quilty", "Quin", "Quinlan", "Quinlivan", "Quinn", "Quinney", "Quinny", "Quirke", "Rabbitte",
        "Rafferty", "Rafter", "Raftery", "Raftis", "Rahilly", "Raight", "Rails", "Raleigh", "Randles", "Raney", "Raol",
        "Rattigan", "Rawley", "Rayel", "Rea", "Reade", "Reardon", "Reavy", "Reddin", "Reddy", "Redican", "Redmond",
        "Reen", "Regan", "Reid", "Reidy", "Reilly", "Renehan", "Reynell", "Reynolds", "Reynoldson", "Rhatigan",
        "Rhattigan", "Rice", "Richard", "Richards", "Richardson", "Richey", "Richie", "Ridge", "Rigney", "Riney",
        "Ring", "Rinn", "Riordan", "Roach", "Roache", "Roarke", "Roarty", "Roberts", "Robertson", "Robeson", "Robinson",
        "Roche", "Rock", "Rodden", "Roddy", "Roden", "Rodgers", "Roe", "Rogers", "Rogerson", "Rohan", "Roland", "Ronan",
        "Ronayne", "Rooney", "Rose", "Ross", "Rourke", "Rowan", "Rowe", "Rowley", "Ruane", "Rudden", "Ruddy", "Rudkins",
        "Rush", "Russell", "Ryan", "Ryder", "Ryle", "Rynn", "Rynne", "Salmon", "Sammon", "Saors", "Sarsfield", "Sayers",
        "Scallan", "Scallon", "Scally", "Scanlan", "Scanlon", "Scannell", "Scollan", "Scriven", "Scullion", "Scully",
        "Seally", "Sealy", "Sears", "Seery", "Segerson", "Segersun", "Setrick", "Sexton", "Shaffrey", "Shanahan",
        "Shanley", "Shannon", "Shanny", "Sharkey", "Sharpe", "Sharry", "Shaughnessy", "Shea", "Sheahan", "Sheane",
        "Sheedy", "Sheehan", "Sheehy", "Sheeran", "Sheerin", "Sheil", "Sheilds", "Sheridan", "Sherlock", "Sherry",
        "Shevlin", "Shiel", "Shields", "Shiels", "Shine", "Short", "Shortt", "Sigerson", "Silk", "Silke", "Simmon",
        "Simmonds", "Simmons", "Sinan", "Sinnott", "Skally", "Skeahan", "Skeffington", "Skehan", "Skelly", "Skivington",
        "Slamon", "Slattery", "Slevin", "Sloan", "Sloane", "Slowey", "Slyne", "Small", "Smith", "Smullen", "Smyth",
        "Smythe", "Somers", "Soolaghan", "Spain", "Spencer", "Spenser", "Spillane", "Stack", "Stanton", "Stapleton",
        "Staunton", "Steed", "Stenson", "Stephens", "Stephenson", "Steward", "Stewart", "Stoices", "Stokes", "Stone",
        "Storey", "Story", "Stuart", "Sugrue", "Sullivan", "Summerville", "Supple", "Sweeney", "Sweeny", "Swift",
        "Swords", "Synnott", "Taggart", "Tangney", "Tansey", "Tarpey", "Taylor", "Teahan", "Tehan", "Ternan", "Terry",
        "Thom", "Thomas", "Thompson", "Thornton", "Tiernan", "Tierney", "Timlin", "Timoney", "Timony", "Tinney", "Toal",
        "Tobin", "Togher", "Tohall", "Tolan", "Tolin", "Toms", "Toner", "Toolan", "Toole", "Toolin", "Toolis", "Tooman",
        "Toomey", "Tormay", "Tormey", "Torpey", "Torrence", "Torrens", "Tracey", "Tracy", "Trainor", "Travers",
        "Traynor", "Treacy", "Treanor", "Trenor", "Troy", "Tubridy", "Tully", "Tuohey", "Tuohy", "Turley", "Tutty",
        "Twohey", "Twohig", "Twomey", "Tynan", "Tyrrell", "Uniacke", "Uniaque", "Vaughan", "Veale", "Victory", "Wade",
        "Waldron", "Wall", "Wallace", "Walls", "Walsh", "Walshe", "Walter", "Walters", "Ward", "Warren", "Waters",
        "Watters", "Watts", "Weaver", "Weever", "Weir", "Weldon", "Whalen", "Whelan", "Whelehan", "White", "Whitty",
        "Whyte", "Wilkins", "Wilkinson", "Williams", "Wilson", "Winters", "Wolfe", "Woods", "Woolley", "Woulfe", "Wren",
        "Wrenn", "Wright", "Wrynn", "Wynne", "Young", "de Courcey", "de Lacy", "Ó Corra",
    )

    prefixes_female = ('Mrs.', 'Ms.', 'Miss', 'Dr.')
    prefixes_male = ('Mr.', 'Dr.')



from .. import Provider as CurrencyProvider


class Provider(CurrencyProvider):

    price_formats = ["#.##", "%#.##", "%##.##", "%,###.##", "%#,###.##"]

    def pricetag(self):
        return ("$" + self.numerify(self.random_element(self.price_formats)))



from .. import Provider as BaseProvider


class Provider(BaseProvider):
    INVALID_SSN_TYPE = 'INVALID_SSN'
    SSN_TYPE = 'SSN'
    ITIN_TYPE = 'ITIN'
    EIN_TYPE = 'EIN'

    def itin(self):
        """Generate a random United States Individual Taxpayer Identification Number (ITIN).

        An United States Individual Taxpayer Identification Number
        (ITIN) is a tax processing number issued by the Internal
        Revenue Service. It is a nine-digit number that always begins
        with the number 9 and has a range of 70-88 in the fourth and
        fifth digit. Effective April 12, 2011, the range was extended
        to include 900-70-0000 through 999-88-9999, 900-90-0000
        through 999-92-9999 and 900-94-0000 through 999-99-9999.
        https://www.irs.gov/individuals/international-taxpayers/general-itin-information
        """

        area = self.random_int(min=900, max=999)
        serial = self.random_int(min=0, max=9999)

        # The group number must be between 70 and 99 inclusively but not 89 or 93
        group = self.random_element([x for x in range(70, 100) if x not in [89, 93]])

        itin = f'{area:03d}-{group:02d}-{serial:04d}'
        return itin

    def ein(self):
        """Generate a random United States Employer Identification Number (EIN).

         An United States An Employer Identification Number (EIN) is
         also known as a Federal Tax Identification Number, and is
         used to identify a business entity. EINs follow a format of a
         two-digit prefix followed by a hyphen and a seven-digit sequence:
         ##-######

         https://www.irs.gov/businesses/small-businesses-self-employed/employer-id-numbers
        """

        # Only certain EIN Prefix values are assigned:
        #
        # https://www.irs.gov/businesses/small-businesses-self-employed/how-eins-are-assigned-and-valid-ein-prefixes

        ein_prefix_choices = [
            '01',
            '02',
            '03',
            '04',
            '05',
            '06',
            '10',
            '11',
            '12',
            '13',
            '14',
            '15',
            '16',
            '20',
            '21',
            '22',
            '23',
            '24',
            '25',
            '26',
            '27',
            '30',
            '31',
            '32',
            '33',
            '34',
            '35',
            '36',
            '37',
            '38',
            '39',
            '40',
            '41',
            '42',
            '43',
            '44',
            '45',
            '46',
            '47',
            '48',
            '50',
            '51',
            '52',
            '53',
            '54',
            '55',
            '56',
            '57',
            '58',
            '59',
            '60',
            '61',
            '62',
            '63',
            '64',
            '65',
            '66',
            '67',
            '68',
            '71',
            '72',
            '73',
            '74',
            '75',
            '76',
            '77',
            '80',
            '81',
            '82',
            '83',
            '84',
            '85',
            '86',
            '87',
            '88',
            '90',
            '91',
            '92',
            '93',
            '94',
            '95',
            '98',
            '99']

        ein_prefix = self.random_element(ein_prefix_choices)
        sequence = self.random_int(min=0, max=9999999)

        ein = f'{ein_prefix:s}-{sequence:07d}'
        return ein

    def invalid_ssn(self):
        """ Generate a random invalid United States Social Security Identification Number (SSN).

        Invalid SSNs have the following characteristics:
        Cannot begin with the number 9
        Cannot begin with 666 in positions 1 - 3
        Cannot begin with 000 in positions 1 - 3
        Cannot contain 00 in positions 4 - 5
        Cannot contain 0000 in positions 6 - 9

        https://www.ssa.gov/kc/SSAFactSheet--IssuingSSNs.pdf

        Additionally, return an invalid SSN that is NOT a valid ITIN by excluding certain ITIN related "group" values
        """
        itin_group_numbers = [
            70,
            71,
            72,
            73,
            74,
            75,
            76,
            77,
            78,
            79,
            80,
            81,
            82,
            83,
            84,
            85,
            86,
            87,
            88,
            90,
            91,
            92,
            94,
            95,
            96,
            97,
            98,
            99]
        area = self.random_int(min=0, max=999)
        if area < 900 and area not in {666, 0}:
            random_group_or_serial = self.random_int(min=1, max=1000)
            if random_group_or_serial <= 500:
                group = 0
                serial = self.random_int(0, 9999)
            else:
                group = self.random_int(0, 99)
                serial = 0
        elif area in {666, 0}:
            group = self.random_int(0, 99)
            serial = self.random_int(0, 9999)
        else:
            group = self.random_element([x for x in range(0, 100) if x not in itin_group_numbers])
            serial = self.random_int(0, 9999)

        invalid_ssn = f'{area:03d}-{group:02d}-{serial:04d}'
        return invalid_ssn

    def ssn(self, taxpayer_identification_number_type=SSN_TYPE):
        """ Generate a random United States Taxpayer Identification Number of the specified type.

        If no type is specified, a US SSN is returned.
        """

        if taxpayer_identification_number_type == self.ITIN_TYPE:
            return self.itin()
        elif taxpayer_identification_number_type == self.EIN_TYPE:
            return self.ein()
        elif taxpayer_identification_number_type == self.INVALID_SSN_TYPE:
            return self.invalid_ssn()
        elif taxpayer_identification_number_type == self.SSN_TYPE:

            # Certain numbers are invalid for United States Social Security
            # Numbers. The area (first 3 digits) cannot be 666 or 900-999.
            # The group number (middle digits) cannot be 00. The serial
            # (last 4 digits) cannot be 0000.

            area = self.random_int(min=1, max=899)
            if area == 666:
                area += 1
            group = self.random_int(1, 99)
            serial = self.random_int(1, 9999)

            ssn = f'{area:03d}-{group:02d}-{serial:04d}'
            return ssn

        else:
            raise ValueError("taxpayer_identification_number_type must be one of 'SSN', 'EIN', 'ITIN',"
                             " or 'INVALID_SSN'.")



from .. import Provider as PersonProvider


class Provider(PersonProvider):

    formats_male = (
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}-{{last_name}}',
        '{{prefix_male}} {{first_name_male}} {{last_name}}',
    )

    formats_female = (
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}-{{last_name}}',
        '{{prefix_female}} {{first_name_female}} {{last_name}}',
    )

    formats = formats_male + formats_female

    # source: https://en.wikipedia.org/w/index.php?title=Category:Italian_masculine_given_names
    first_names_male = (
        'Achille', 'Adamo', 'Adelmo', 'Adriano', 'Agnolo', 'Agostino',
        'Alberico', 'Alberto', 'Alderano', 'Aldo', 'Alessandro', 'Alessio', 'Alfio',
        'Alfredo', 'Alphons', 'Amadeo', 'Amedeo', 'Amico', 'Amleto', 'Angelo', 'Annibale',
        'Ansaldo', 'Antonello', 'Antonino', 'Antonio', 'Armando', 'Arnaldo', 'Arnulfo',
        'Arsenio', 'Arturo', 'Atenulf', 'Augusto', 'Azeglio', 'Baccio', 'Baldassare',
        'Bartolomeo', 'Benedetto', 'Benito', 'Benvenuto', 'Beppe', 'Bernardo', 'Biagio',
        'Bruno', 'Calcedonio', 'Calogero', 'Camillo', 'Carlo', 'Carmelo', 'Cesare',
        'Cipriano', 'Cirillo', 'Ciro', 'Claudio', 'Coluccio', 'Coriolano', 'Corrado',
        'Costantino', 'Costanzo', 'Damiano', 'Daniele', 'Danilo', 'Dante', 'Dario', 'Delfino',
        'Dino', 'Dionigi', 'Domenico', 'Donatello', 'Donato', 'Durante', 'Edoardo',
        'Elladio', 'Elmo', 'Emilio', 'Ennio', 'Enrico', 'Enzio', 'Enzo', 'Eraldo',
        'Ermanno', 'Ermenegildo', 'Ermes', 'Ernesto', 'Ettore', 'Ezio', 'Fabio', 'Fabrizio',
        'Fausto', 'Fedele', 'Federico', 'Federigo', 'Ferdinando', 'Filippo', 'Fiorenzo',
        'Fiorino', 'Flavio', 'Francesco', 'Franco', 'Fredo', 'Fulvio', 'Gabriele',
        'Gaetano', 'Galasso', 'Gaspare', 'Gastone', 'Geronimo', 'Giacinto', 'Giacobbe',
        'Giacomo', 'Giampaolo', 'Giampiero', 'Gian', 'Giancarlo', 'Gianfrancesco',
        'Gianfranco', 'Gianluca', 'Gianluigi', 'Gianmarco', 'Gianni', 'Gianpaolo',
        'Gianpietro', 'Gilberto', 'Gino', 'Gioacchino', 'Gioachino', 'Gioele', 'Gioffre',
        'Gionata', 'Giorgio', 'Giosuè', 'Giovanni', 'Girolamo', 'Giuliano', 'Giulio',
        'Giuseppe', 'Giustino', 'Goffredo', 'Graziano', 'Greco', 'Guarino', 'Guglielmo',
        'Guido', 'Gustavo', 'Hugo', 'Ignazio', 'Ippazio', 'Ivan', 'Ivo', 'Jacopo',
        'Lamberto', 'Lando', 'Laureano', 'Lazzaro', 'Leonardo', 'Leone', 'Leopoldo',
        'Liberto', 'Livio', 'Lodovico', 'Lorenzo', 'Luca', 'Luchino', 'Luciano', 'Lucio',
        'Ludovico', 'Luigi', 'Manuel', 'Marcantonio', 'Marcello', 'Marco', 'Mariano',
        'Mario', 'Martino', 'Martino', 'Massimiliano', 'Massimo', 'Matteo', 'Mattia',
        'Maurilio', 'Maurizio', 'Mauro', 'Michelangelo', 'Michele', 'Micheletto',
        'Michelotto', 'Milo', 'Mirco', 'Mirko', 'Nanni', 'Napoleone', 'Niccolò', 'Nico',
        'Nicola', 'Nicolò', 'Nino', 'Orazio', 'Oreste', 'Orlando', 'Osvaldo', 'Ottavio',
        'Ottone', 'Panfilo', 'Paolo', 'Paride', 'Pasqual', 'Pasquale', 'Patrizio',
        'Pellegrino', 'Pier', 'Pierangelo', 'Piergiorgio', 'Piergiuseppe', 'Pierluigi',
        'Piermaria', 'Piero', 'Pierpaolo', 'Piersanti', 'Pietro', 'Pompeo', 'Pomponio',
        'Puccio', 'Raffaele', 'Raffaellino', 'Raffaello', 'Raimondo', 'Ranieri',
        'Rembrandt', 'Renzo', 'Riccardo', 'Ricciotti', 'Roberto', 'Rocco', 'Rodolfo',
        'Rolando', 'Roman', 'Romeo', 'Romolo', 'Ronaldo', 'Rosario', 'Ruggero', 'Ruggiero',
        'Sabatino', 'Salvatore', 'Salvi', 'Sandro', 'Sante', 'Santino', 'Saverio',
        'Sebastiano', 'Sergius', 'Severino', 'Silvestro', 'Silvio', 'Simone', 'Stefano',
        'Telemaco', 'Temistocle', 'Tiziano', 'Toni', 'Tonino', 'Torquato', 'Tullio',
        'Ubaldo', 'Uberto', 'Ugo', 'Ugolino', 'Umberto', 'Valerio', 'Venancio',
        'Vincentio', 'Vincenzo', 'Virgilio', 'Vito', 'Vittorio',
    )
    # source: https://en.wikipedia.org/wiki/Category:Italian_feminine_given_names
    first_names_female = (
        'Adelasia', 'Adele', 'Adriana', 'Alessandra', 'Alessia',
        'Alina', 'Allegra', 'Amalia', 'Amanda', 'Angelica', 'Angelina', 'Anita',
        'Annalisa', 'Annamaria', 'Annetta', 'Annunziata', 'Antonella', 'Antonia',
        'Antonietta', 'Antonina', 'Aria', 'Aurora', 'Barbara', 'Beatrice', 'Berenice',
        'Bettina', 'Bianca', 'Bianca', 'Camilla', 'Carla', 'Carolina', 'Cassandra',
        'Caterina', 'Cecilia', 'Chiara', 'Claudia', 'Clelia', 'Concetta', 'Cristina',
        'Daria', 'Diana', 'Dina', 'Dolores', 'Donatella', 'Donna', 'Eleanora', 'Elena',
        'Eliana', 'Elisa', 'Elvira', 'Emma', 'Erika', 'Etta', 'Eugenia', 'Eva',
        'Evangelista', 'Fabia', 'Fabrizia', 'Federica', 'Fernanda', 'Fiamma', 'Filippa',
        'Flavia', 'Flora', 'Fortunata', 'Francesca', 'Gabriella', 'Gelsomina', 'Gemma',
        'Germana', 'Giada', 'Gianna', 'Giorgia', 'Giovanna', 'Giulia', 'Giuliana',
        'Giulietta', 'Giuseppina', 'Gloria', 'Graziella', 'Greca', 'Griselda', 'Ida',
        'Ilaria', 'Imelda', 'Iolanda', 'Irma', 'Isa', 'Isabella', 'Jolanda', 'Lara',
        'Laura', 'Lauretta', 'Letizia', 'Liana', 'Licia', 'Lidia', 'Liliana', 'Lilla',
        'Lina', 'Lisa', 'Livia', 'Lolita', 'Loredana', 'Loretta', 'Lucia', 'Luciana',
        'Lucrezia', 'Ludovica', 'Luigina', 'Luisa', 'Marcella', 'Margherita', 'Maria',
        'Maria', 'Maria', 'Mariana', 'Marina', 'Marisa', 'Marissa', 'Marta', 'Martina',
        'Matilda', 'Maura', 'Melania', 'Melina', 'Melissa', 'Mercedes', 'Michela', 'Milena',
        'Monica', 'Morena', 'Nadia', 'Natalia', 'Nedda', 'Nicoletta', 'Nina', 'Ninetta',
        'Olga', 'Ornella', 'Paloma', 'Paola', 'Paoletta', 'Patrizia', 'Paulina',
        'Pierina', 'Pina', 'Priscilla', 'Raffaella', 'Ramona', 'Renata', 'Rita', 'Roberta',
        'Romana', 'Romina', 'Rosa', 'Rosalia', 'Rosaria', 'Rosina', 'Rossana', 'Sandra',
        'Serafina', 'Serena', 'Silvia', 'Simonetta', 'Sole', 'Sonia', 'Sophia', 'Stefani',
        'Stefania', 'Stella', 'Susanna', 'Sylvia', 'Tatiana', 'Teresa', 'Tina', 'Tiziana',
        'Tonia', 'Valentina', 'Valeria', 'Vanessa', 'Veronica', 'Victoria',
        'Vincenza', 'Virginia', 'Viridiana', 'Vittoria', 'Zaira',
    )

    first_names = first_names_male + first_names_female

    # source: https://en.wiktionary.org/w/index.php?title=Category:Italian_surnames
    last_names = (
        'Abatantuono', 'Abate', 'Abba', 'Abbagnale', 'Accardo', 'Acerbi',
        'Adinolfi', 'Agazzi', 'Agnesi', 'Agostinelli', 'Agostini', 'Ajello',
        'Albertini', 'Alboni', 'Aldobrandi', 'Alfieri', 'Alfonsi', 'Alighieri', 'Almagià',
        'Aloisio', 'Alonzi', 'Altera', 'Amaldi', 'Amato', 'Ammaniti', 'Anastasi',
        'Andreotti', 'Andreozzi', 'Angeli', 'Angiolello', 'Anguillara', 'Anguissola',
        'Anichini', 'Antelami', 'Antonacci', 'Antonelli', 'Antonello', 'Antonetti',
        'Antonini', 'Antonioni', 'Antonucci', 'Aporti', 'Argan', 'Argentero', 'Argenti',
        'Argento', 'Argurio', 'Ariasso', 'Ariosto', 'Armani', 'Armellini', 'Asmundo',
        'Asprucci', 'Aulenti', 'Avogadro', 'Babati', 'Babato', 'Babbo', 'Bacosi', 'Badoer',
        'Badoglio', 'Baggio', 'Baglioni', 'Bajamonti', 'Bajardi', 'Balbi', 'Balbo', 'Balla',
        'Balotelli', 'Bandello', 'Baracca', 'Barbarigo', 'Barberini', 'Barcaccia', 'Barcella',
        'Barese', 'Baresi', 'Barillaro', 'Baroffio', 'Barozzi', 'Barracco', 'Barsanti',
        'Bartoli', 'Barzini', 'Basadonna', 'Bassi', 'Basso', 'Bataglia', 'Battaglia',
        'Battelli', 'Battisti', 'Bazzi', 'Beccaria', 'Beccheria', 'Beffa', 'Belletini',
        'Bellini', 'Bellocchio', 'Bellucci', 'Bellò', 'Bembo', 'Benedetti', 'Benigni',
        'Benussi', 'Berengario', 'Bergoglio', 'Berlusconi', 'Bernardi', 'Bernardini',
        'Bernetti', 'Bernini', 'Berrè', 'Bersani', 'Bertoli', 'Bertolucci', 'Bertoni',
        'Bettin', 'Bettoni', 'Bevilacqua', 'Biagi', 'Biagiotti', 'Bianchi', 'Bianchini',
        'Bignami', 'Bignardi', 'Binaghi', 'Bixio', 'Blasi', 'Boaga', 'Bocca', 'Boccaccio',
        'Boccherini', 'Boccioni', 'Bocelli', 'Bodoni', 'Boezio', 'Boiardo', 'Boitani', 'Boito',
        'Boldù', 'Bombieri', 'Bompiani', 'Bonanno', 'Bonatti', 'Bonaventura', 'Bondumier',
        'Bongiorno', 'Bonino', 'Bonolis', 'Bonomo', 'Borghese', 'Borgia', 'Borrani',
        'Borromeo', 'Borromini', 'Borroni', 'Borsellino', 'Borsiere', 'Borzomì', 'Bosio',
        'Bossi', 'Bosurgi', 'Botta', 'Bottaro', 'Botticelli', 'Bottigliero', 'Bova',
        'Bragadin', 'Bragaglia', 'Bramante', 'Brambilla', 'Brancaccio', 'Branciforte',
        'Brenna', 'Bresciani', 'Briccialdi', 'Brichese', 'Broggini', 'Broschi', 'Brugnaro',
        'Brunelleschi', 'Brunello', 'Bruno', 'Bruscantini', 'Bulzoni', 'Buonauro', 'Burcardo',
        'Buscetta', 'Busoni', 'Cabibbo', 'Caboto', 'Cabrini', 'Caccianemico', 'Caccioppoli',
        'Cadorna', 'Caetani', 'Cafarchia', 'Caffarelli', 'Cagnin', 'Cagnotto', 'Cainero',
        'Caironi', 'Calarco', 'Calbo', 'Calgari', 'Callegari', 'Callegaro', 'Calvo',
        'Camanni', 'Camicione', 'Camilleri', 'Camiscione', 'Cammarata', 'Campanella',
        'Campano', 'Campise', 'Camuccini', 'Canali', 'Canetta', 'Canevascini', 'Canil',
        'Cannizzaro', 'Canova', 'Cantimori', 'Capecchi', 'Capone', 'Cappelli', 'Capuana',
        'Caracciolo', 'Cardano', 'Carducci', 'Carfagna', 'Carli', 'Carnera', 'Carocci',
        'Carosone', 'Carpaccio', 'Carriera', 'Carullo', 'Caruso', 'Casadei', 'Casagrande',
        'Casale', 'Casaleggio', 'Casalodi', 'Casarin', 'Casellati', 'Casini', 'Cassarà',
        'Castelli', 'Castellitto', 'Castiglione', 'Castioni', 'Catalano', 'Catenazzi',
        'Cattaneo', 'Cavalcanti', 'Cavanna', 'Ceci', 'Celentano', 'Cendron', 'Ceravolo',
        'Ceri', 'Cerquiglini', 'Cerutti', 'Cesaroni', 'Cesarotti', 'Ceschi', 'Chechi',
        'Cheda', 'Cherubini', 'Chiappetta', 'Chiaramonte', 'Chiesa', 'Chigi', 'Chindamo',
        'Chinnici', 'Chittolini', 'Ciampi', 'Cianciolo', 'Ciani', 'Cibin', 'Cicala',
        'Cicilia', 'Cignaroli', 'Cilea', 'Cilibrasi', 'Cimarosa', 'Cimini', 'Cipolla',
        'Civaschi', 'Coardi', 'Cocci', 'Cociarelli', 'Colletti', 'Collina', 'Collodi',
        'Columbo', 'Combi', 'Comboni', 'Comencini', 'Comeriato', 'Comisso', 'Comolli',
        'Condoleo', 'Contarini', 'Conte', 'Conti', 'Contrafatto', 'Coppola', 'Corbo',
        'Corcos', 'Corradi', 'Correr', 'Cortese', 'Cossiga', 'Costalonga', 'Costanzi',
        'Cremonesi', 'Crespi', 'Crisafulli', 'Crispi', 'Cristoforetti', 'Cuda', 'Cugia',
        'Cundari', 'Cuomo', 'Curatoli', 'Curci', 'Curiel', 'Cusano', 'Cutrufo', 'Cutuli',
        'Cuzzocrea', 'Dalla', 'Dallapé', 'Dallara', 'Dandolo', 'Deledda', 'Delle', 'Dellucci',
        'Depero', 'Desio', 'Detti', 'Dibiasi', 'Disdero', 'Doglioni', 'Donarelli',
        'Donati', 'Donatoni', 'Donini', 'Donà', 'Doria', 'Dossetti', 'Dossi', 'Dovara',
        'Draghi', 'Druso', 'Dulbecco', 'Duodo', 'Durante', 'Duse', 'Eco',
        'Einaudi', 'Emanuelli', 'Emo', 'Endrizzi', 'Errani', 'Errigo', 'Esposito', 'Fabbri',
        'Fabrizi', 'Faggiani', 'Fagiani', 'Fagotto', 'Falcone', 'Falier', 'Fallaci',
        'Falloppio', 'Fantini', 'Fantoni', 'Fantozzi', 'Fanucci', 'Faranda', 'Farina',
        'Farinelli', 'Farnese', 'Fattori', 'Faugno', 'Favata', 'Federici', 'Federico',
        'Fermi', 'Ferrabosco', 'Ferragamo', 'Ferragni', 'Ferrante', 'Ferrara', 'Ferrari',
        'Ferraris', 'Ferrata', 'Ferrazzi', 'Ferretti', 'Ferrucci', 'Fibonacci', 'Ficino',
        'Fieramosca', 'Filangieri', 'Filippelli', 'Filippini', 'Filogamo', 'Filzi', 'Finetti',
        'Finotto', 'Finzi', 'Fioravanti', 'Fiorucci', 'Fischetti', 'Fittipaldi', 'Flaiano',
        'Florio', 'Fo', 'Foa', 'Foconi', 'Fogazzaro', 'Foletti', 'Folliero',
        'Fornaciari', 'Forza', 'Foscari', 'Foà', 'Fracci', 'Franceschi', 'Franscini',
        'Franzese', 'Frescobaldi', 'Fusani', 'Fuseli', 'Gabba', 'Gabbana', 'Gabrieli',
        'Gadda', 'Gaggini', 'Gagliano', 'Gagliardi', 'Gaiatto', 'Gaito', 'Galeati',
        'Galiazzo', 'Galilei', 'Galtarossa', 'Galuppi', 'Galvani', 'Gangemi', 'Gargallo',
        'Garibaldi', 'Garobbio', 'Garozzo', 'Garrone', 'Garzoni', 'Gasperi', 'Gatto', 'Gelli',
        'Gemito', 'Gentileschi', 'Gentili', 'Gentilini', 'Geraci', 'Germano', 'Giacconi',
        'Giacometti', 'Giammusso', 'Gianetti', 'Gianinazzi', 'Giannelli', 'Giannetti',
        'Giannini', 'Giannone', 'Giannotti', 'Giannuzzi', 'Gianvecchio', 'Gibilisco',
        'Gigli', 'Gilardoni', 'Ginese', 'Ginesio', 'Gioberti', 'Giolitti', 'Giorgetti',
        'Giovine', 'Giradello', 'Giulietti', 'Giunti', 'Giusti', 'Goldoni', 'Goldstein',
        'Golgi', 'Golino', 'Gonzaga', 'Gori', 'Gottardi', 'Gotti', 'Govoni', 'Gozzano',
        'Gozzi', 'Gradenigo', 'Gramsci', 'Granatelli', 'Grassi', 'Grasso', 'Gravina',
        'Greco', 'Greggio', 'Gregori', 'Gregorio', 'Gremese', 'Grifeo', 'Grimani',
        'Grisoni', 'Gritti', 'Grossi', 'Gualandi', 'Gualtieri', 'Guarana', 'Guarato',
        'Guariento', 'Guarneri', 'Gucci', 'Guglielmi', 'Guicciardini', 'Guidone', 'Guidotti',
        'Guinizzelli', 'Gullotta', 'Gulotta', 'Gussoni', 'Iacobucci', 'Iacovelli', 'Iadanza',
        'Iannelli', 'Iannotti', 'Iannucci', 'Iannuzzi', 'Impastato', 'Infantino',
        'Innocenti', 'Interiano', 'Interminei', 'Interminelli', 'Inzaghi', 'Ioppi', 'Jacuzzi',
        'Jilani', 'Jovinelli', 'Juvara', 'Lamborghini', 'Lancisi',
        'Lanfranchi', 'Lattuada', 'Leblanc', 'Legnante', 'Leonardi', 'Leoncavallo', 'Leone',
        'Leonetti', 'Leopardi', 'Lercari', 'Lerner', 'Letta', 'Lettiere', 'Ligorio',
        'Liguori', 'Lippomano', 'Littizzetto', 'Liverotti', 'Lollobrigida',
        'Lombardi', 'Lombardo', 'Lombroso', 'Longhena', 'Lopresti', 'Loredan', 'Lovato',
        'Lucarelli', 'Lucchesi', 'Lucciano', 'Luciani', 'Ludovisi', 'Luna', 'Lupo', 'Luria',
        'Lussu', 'Luxardo', 'Luzi', 'Maccanelli', 'Maderna', 'Maderno', 'Maffei',
        'Maggioli', 'Maglio', 'Magnani', 'Magrassi', 'Majewski', 'Majorana', 'Malacarne',
        'Malaparte', 'Malatesta', 'Malenchini', 'Malipiero', 'Malpighi', 'Manacorda',
        'Mancini', 'Mannoia', 'Manolesso', 'Mantegazza', 'Mantegna', 'Manunta', 'Manzoni',
        'Marangoni', 'Marazzi', 'Marcacci', 'Marconi', 'Marenzio', 'Marinetti', 'Marini',
        'Marino', 'Marrone', 'Marsili', 'Martinelli', 'Martucci', 'Marzorati', 'Mascagni',
        'Mascheroni', 'Maspero', 'Mastandrea', 'Mastroianni', 'Mattarella', 'Matteotti',
        'Mazzacurati', 'Mazzanti', 'Mazzeo', 'Mazzi', 'Mazzini', 'Mazzocchi', 'Medici',
        'Mengolo', 'Mennea', 'Mercadante', 'Mercalli', 'Mercantini', 'Mercati', 'Merisi',
        'Metella', 'Meucci', 'Mezzetta', 'Micca', 'Michelangeli', 'Micheletti',
        'Migliaccio', 'Milanesi', 'Mimun', 'Miniati', 'Missoni', 'Moccia', 'Mocenigo',
        'Modiano', 'Modigliani', 'Modugno', 'Mogherini', 'Molesini', 'Monaco', 'Mondadori',
        'Mondaini', 'Monduzzi', 'Moneta', 'Monicelli', 'Montalcini', 'Montalti', 'Montanari',
        'Montanariello', 'Montanelli', 'Monte', 'Montecchi', 'Montesano', 'Montessori',
        'Monteverdi', 'Monti', 'Morabito', 'Morandi', 'Morandini', 'Morellato', 'Moresi',
        'Moretti', 'Morgagni', 'Morlacchi', 'Morosini', 'Morpurgo', 'Morricone', 'Morrocco',
        'Mortati', 'Morucci', 'Moschino', 'Mozart', 'Munari', 'Muratori', 'Murialdo',
        'Murri', 'Musatti', 'Mussolini', 'Muti', 'Naccari', 'Nadi', 'Napolitano', 'Natta',
        'Navarria', 'Navone', 'Necci', 'Nibali', 'Nicoletti', 'Nicolini', 'Nicolucci',
        'Nievo', 'Niggli', 'Niscoromni', 'Nitti', 'Nitto', 'Nolcini', 'Nonis', 'Norbiato',
        'Nordio', 'Nosiglia', 'Notarbartolo', 'Novaro', 'Nugnes', 'Odescalchi', 'Offredi',
        'Oliboni', 'Olivetti', 'Omma', 'Onio', 'Onisto', 'Opizzi', 'Orengo', 'Orlando',
        'Orsini', 'Ortese', 'Ortolani', 'Oscuro', 'Ossani', 'Ossola', 'Ostinelli',
        'Ottino', 'Ovadia', 'Pace', 'Pacelli', 'Pacetti', 'Pacillo', 'Pacomio', 'Padovano',
        'Paganini', 'Pagliaro', 'Pagnotto', 'Palazzo', 'Palladio', 'Palmisano', 'Palombi',
        'Paltrinieri', 'Palumbo', 'Panatta', 'Panicucci', 'Panzera', 'Paoletti', 'Paolini',
        'Paolucci', 'Papafava', 'Papetti', 'Pareto', 'Parini', 'Parisi', 'Parmitano',
        'Parpinel', 'Parri', 'Paruta', 'Pascarella', 'Pasolini', 'Pasqua', 'Passalacqua',
        'Pastine', 'Pausini', 'Pavanello', 'Pavarotti', 'Pavone', 'Peano', 'Pederiva',
        'Pedersoli', 'Pedrazzini', 'Pedroni', 'Pellegrini', 'Pelli', 'Pellico', 'Pennetta',
        'Pepe', 'Peranda', 'Pergolesi', 'Perini', 'Perozzo', 'Persico', 'Pertile',
        'Pertini', 'Peruzzi', 'Petralli', 'Petrassi', 'Petrocelli', 'Petrucci',
        'Petrucelli', 'Petruzzi', 'Pezzali', 'Piacentini', 'Piane', 'Piazzi', 'Piccinni',
        'Piccio', 'Pietrangeli', 'Pigafetta', 'Pignatti', 'Pinamonte', 'Pincherle',
        'Pininfarina', 'Piovani', 'Pirandello', 'Pirelli', 'Pisacane', 'Pisani',
        'Pisano', 'Pisaroni', 'Pistoletto', 'Pizzamano', 'Pizzetti', 'Pizziol', 'Pizzo',
        'Platini', 'Poerio', 'Polani', 'Polesel', 'Polizzi', 'Pometta', 'Pontecorvo',
        'Ponti', 'Porcellato', 'Porzio', 'Pozzecco', 'Prada', 'Praga', 'Pratesi', 'Prati',
        'Priuli', 'Procacci', 'Prodi', 'Proietti', 'Pucci', 'Puccini', 'Pugliese',
        'Puglisi', 'Pulci', 'Quasimodo', 'Querini', 'Raimondi', 'Ramazzotti', 'Randazzo',
        'Rapisardi', 'Rastelli', 'Raurica', 'Ravaglioli', 'Redi', 'Regge', 'Renault',
        'Renier', 'Rensi', 'Renzi', 'Respighi', 'Riccardi', 'Riccati', 'Ricci',
        'Ricciardi', 'Ricolfi', 'Rienzo', 'Righi', 'Rinaldi', 'Rismondo', 'Ritacca', 'Rizzo',
        'Rizzoli', 'Rocca', 'Roccabonella', 'Roero', 'Romagnoli', 'Romano', 'Romiti',
        'Roncalli', 'Rosiello', 'Rosmini', 'Rosselli', 'Rossellini', 'Rossetti', 'Rossi',
        'Rossini', 'Roth', 'Rubbia', 'Ruberto', 'Ruffini', 'Ruggeri', 'Ruggieri', 'Russo',
        'Rusticucci', 'Sabatini', 'Sabbatini', 'Saffi', 'Sagese', 'Sagnelli', 'Sagredo',
        'Salandra', 'Salata', 'Salgari', 'Salieri', 'Salvemini', 'Salvini', 'Salvo',
        'Samele', 'Sandi', 'Sanguineti', 'Sansoni', 'Santi', 'Santorio', 'Santoro',
        'Sanudo', 'Saraceno', 'Saracino', 'Saragat', 'Satriani', 'Satta', 'Sauli', 'Sauro',
        'Savorgnan', 'Sbarbaro', 'Scaduto', 'Scalera', 'Scalfaro', 'Scamarcio', 'Scandone',
        'Scaramucci', 'Scarfoglio', 'Scarlatti', 'Scarpa', 'Scarpetta', 'Scarponi',
        'Schiaparelli', 'Schiavo', 'Schiavone', 'Schicchi', 'Scialpi', 'Scotti', 'Scotto',
        'Seddio', 'Segni', 'Segrè', 'Semitecolo', 'Serao', 'Serlupi', 'Sermonti',
        'Serraglio', 'Sforza', 'Sgalambro', 'Sgarbi', 'Sibilia', 'Siffredi', 'Silvestri',
        'Simeoni', 'Sinisi', 'Sismondi', 'Smirnoff', 'Sobrero', 'Soderini', 'Soffici',
        'Sokolov', 'Solari', 'Solimena', 'Sollima', 'Sommaruga', 'Sonnino', 'Soprano',
        'Soranzo', 'Sordi', 'Sorrentino', 'Spadafora', 'Spallanzani', 'Spanevello', 'Speri',
        'Spinelli', 'Spinola', 'Squarcione', 'Sraffa', 'Staglieno', 'Stefanelli', 'Stein',
        'Stoppani', 'Storladi', 'Stradivari', 'Strangio', 'Stucchi', 'Surian', 'T',
        'Tacchini', 'Taccola', 'Tafuri', 'Tagliafierro', 'Taliani', 'Taliercio', 'Tamborini',
        'Tamburello', 'Tamburi', 'Tamburini', 'Tanzini', 'Tarantini', 'Tarantino', 'Tarchetti',
        'Tartaglia', 'Tartini', 'Tasca', 'Tasso', 'Tassoni', 'Tebaldi', 'Tedesco', 'Telesio',
        'Tencalla', 'Terragni', 'Tiepolo', 'Tirabassi', 'Togliatti', 'Tognazzi', 'Toldo',
        'Tolentino', 'Tomaselli', 'Tomasetti', 'Tomasini', 'Tomei', 'Tommaseo', 'Toninelli',
        'Tonisto', 'Torlonia', 'Tornatore', 'Torricelli', 'Toscani', 'Toscanini', 'Toselli',
        'Tosi', 'Toso', 'Tosto', 'Totino', 'Tozzi', 'Tozzo', 'Traetta', 'Trapanese',
        'Trapani', 'Travaglia', 'Travaglio', 'Traversa', 'Travia', 'Trebbi', 'Treccani',
        'Tremonti', 'Trentin', 'Trentini', 'Tresoldi', 'Treves', 'Trevisan', 'Trevisani',
        'Trezzini', 'Trillini', 'Trincavelli', 'Trobbiani', 'Troisi', 'Trombetta', 'Tron',
        'Tropea', 'Trotta', 'Trupiano', 'Trussardi', 'Turati', 'Turchetta', 'Turchi',
        'Turci', 'Turrini', 'Tutino', 'Tuzzolino', 'Ubaldi', 'Udinese', 'Udinesi', 'Ughi',
        'Ungaretti', 'Valentino', 'Valguarnera', 'Valier', 'Valmarana', 'Vanvitelli',
        'Varano', 'Vasari', 'Vattimo', 'Vecellio', 'Vecoli', 'Veltroni', 'Vendetti',
        'Venditti', 'Veneziano', 'Venier', 'Vento', 'Venturi', 'Vercelloni', 'Verdi',
        'Verdone', 'Verga', 'Vergassola', 'Vergerio', 'Verri', 'Versace', 'Vespa',
        'Vespucci', 'Vezzali', 'Vianello', 'Vidoni', 'Vigliotti', 'Vigorelli', 'Villadicani',
        'Villarosa', 'Viola', 'Virgilio', 'Visconti', 'Visintini', 'Vismara', 'Vittadello',
        'Vitturi', 'Vivaldi', 'Viviani', 'Volta', 'Volterra', 'Zabarella', 'Zaccagnini',
        'Zaccardo', 'Zacchia', 'Zacco', 'Zaguri', 'Zamengo', 'Zamorani', 'Zampa', 'Zanazzo',
        'Zanichelli', 'Zanzi', 'Zarlino', 'Zecchini', 'Zeffirelli', 'Zetticci', 'Ziani',
        'Zichichi', 'Zito', 'Zola', 'Zoppetti', 'Zoppetto',
        )

    prefixes_female = ('Dott.', 'Sig.ra')
    prefixes_male = ('Dott.', 'Sig.')

    prefixes = ('Dott.', 'Sig.', 'Sig.ra')



from .. import Provider as AddressProvider


class Provider(AddressProvider):
    address_formats = ['{{street_address}}, {{city}}, {{postcode}}']
    building_number_formats = ['#', '##', '###']
    city_formats = ['{{city_prefix}} {{first_name}}']
    street_address_formats = ['{{street_name}}, {{building_number}}']
    street_name_formats = ('{{street_prefix}} {{street_title}}', )

    city_prefixes = ['місто', 'село', 'селище', 'хутір']
    countries = [
        'Австралія', 'Австрія', 'Азербайджан', 'Албанія', 'Алжир', 'Ангола',
        'Андорра', 'Антигуа і Барбуда', 'Аргентина', 'Афганістан',
        'Багамські Острови', 'Бангладеш', 'Барбадос', 'Бахрейн', 'Беліз',
        'Бельгія', 'Бенін', 'Білорусь', 'Болгарія', 'Болівія',
        'Боснія і Герцеговина', 'Ботсвана', 'Бразилія', 'Бруней',
        'Буркіна-Фасо', 'Бурунді', 'Бутан', 'Вануату', 'Ватикан',
        'Велика Британія', 'Венесуела', 'В\'єтнам', 'Вірменія', 'Габон',
        'Гаїті', 'Гаяна', 'Гамбія', 'Гана', 'Гватемала', 'Гвінея',
        'Гвінея-Бісау', 'Гондурас', 'Гренада', 'Греція', 'Грузія', 'Данія',
        'Джибуті', 'Домініка', 'Домініканська Республіка', 'Еквадор',
        'Екваторіальна Гвінея', 'Еритрея', 'Естонія', 'Ефіопія', 'Єгипет',
        'Ємен', 'Замбія', 'Західна Сахара', 'Зімбабве', 'Ізраїль', 'Індія',
        'Індонезія', 'Ірак', 'Іран', 'Ірландія', 'Ісландія', 'Іспанія',
        'Італія', 'Йорданія', 'Кабо-Верде', 'Казахстан', 'Камбоджа', 'Камерун',
        'Канада', 'Катар', 'Кенія', 'Киргизстан', 'КНР', 'Кіпр', 'Кірибаті',
        'Колумбія', 'Коморські Острови', 'Конго', 'ДР Конго', 'Південна Корея',
        'Північна Корея', 'Косово', 'Коста-Рика', 'Кот-д\'Івуар', 'Куба',
        'Кувейт', 'Лаос', 'Латвія', 'Лесото', 'Литва', 'Ліберія', 'Ліван',
        'Лівія', 'Ліхтенштейн', 'Люксембург', 'Маврикій', 'Мавританія',
        'Мадагаскар', 'Республіка Македонія', 'Малаві', 'Малайзія', 'Малі',
        'Мальдіви', 'Мальта', 'Марокко', 'Маршаллові Острови', 'Мексика',
        'Федеративні Штати Мікронезії', 'Мозамбік', 'Молдова', 'Монако',
        'Монголія', 'М\'янма', 'Намібія', 'Науру', 'Непал', 'Нігер', 'Нігерія',
        'Нідерланди', 'Нікарагуа', 'Німеччина', 'Нова Зеландія', 'Норвегія',
        'ОАЕ', 'Оман', 'Пакистан', 'Палау', 'Палестинська держава', 'Панама',
        'Папуа Нова Гвінея', 'ПАР', 'Парагвай', 'Перу', 'Південний Судан',
        'Польща', 'Португалія', 'Росія', 'Руанда', 'Румунія', 'Сальвадор',
        'Самоа', 'Сан-Марино', 'Сан-Томе і Принсіпі', 'Саудівська Аравія',
        'Свазіленд', 'Сейшельські Острови', 'Сенегал',
        'Сент-Вінсент і Гренадини', 'Сент-Кіттс і Невіс', 'Сент-Люсія',
        'Сербія', 'Сінгапур', 'Сирія', 'Словаччина', 'Словенія',
        'Соломонові Острови', 'Сомалі', 'Судан', 'Суринам', 'Східний Тимор',
        'США', 'Сьєрра-Леоне', 'Таджикистан', 'Таїланд', 'Тайвань', 'Танзанія',
        'Того', 'Тонга', 'Тринідад і Тобаго', 'Тувалу', 'Туніс', 'Туреччина',
        'Туркменістан', 'Уганда', 'Угорщина', 'Узбекистан', 'Україна',
        'Уругвай', 'Фіджі', 'Філіппіни', 'Фінляндія', 'Франція', 'Хорватія',
        'Центральноафриканська Республіка', 'Чад', 'Чехія', 'Чилі',
        'Чорногорія', 'Швейцарія', 'Швеція', 'Шрі-Ланка', 'Ямайка', 'Японія',
    ]
    street_prefixes = [
        'вулиця', 'набережна',
    ]
    street_suffixes = ['узвіз']

    street_titles = [
        '40-летия Октября',
        'Академика Шлихтера',
        'Алексея Давыдова',
        'Анищенко',
        'Антонова-Овсеенко',
        'Артема',
        'Бабушкина',
        'Бакинских Комиссаров',
        'Баумана',
        'Блюхера',
        'Боженко',
        'Бонч-Бруевича',
        'Буденного',
        'Ветрова',
        'Воровского',
        'Воссоединения',
        'Гамарника',
        'Горького',
        'Дзержинского',
        'Димитрова',
        'Дубового Ивана',
        'Дундича Олеко',
        'Жданова',
        'Ивана Клименко',
        'Ивана Лепсе',
        'Иванова Андрея',
        'Ильича',
        'Калининская',
        'Киквидзе',
        'Кирова',
        'Коллективизации',
        'Коллонтай',
        'Командарма Уборевич',
        'Комиссара Рыкова',
        'Коммунистическая',
        'Комсомольская',
        'Котовского',
        'Кравченко Николая',
        'Красикова Петра',
        'Красноармейская',
        'Красногвардейская',
        'Краснопартизанская',
        'Краснофлотская',
        'Крупской',
        'Крыленко',
        'Кутузова',
        'Лазо Сергея',
        'Лайоша Гавро',
        'Ластовского',
        'Ленина',
        'Ленинская',
        'Луначарского',
        'Майорова Михаила',
        'Маршала Буденного',
        'Маршала Тухачевского',
        'Мате Залки',
        'Машина Михаила',
        'Мильчакова Александра',
        'Михаила Скрипника',
        'Московская',
        'Октябрьская',
        'Омельяна Горбачова',
        'Островского Николая',
        'Павла Дибенко',
        'Павлика Морозова',
        'Патриса Лумумбы',
        'Перспективная',
        'Петра Дегтяренко',
        'Петра Шелеста',
        'Петровского',
        'Пика Вильгельма',
        'Полупанова',
        'Примакова',
        'Профинтерна',
        'Руднева Николая',
        'Сагайдика Степана',
        'Сарафимовича',
        'Сергея Струтинского',
        'Смирнова-Ласточкина',
        'Советская',
        'Софьи Перовской',
        'Строкача Тимофея',
        'Суворова',
        'Терешковой Валентины',
        'Трутенко Онуфрия',
        'Фадеева',
        'Федько Ивана',
        'Фрунзе',
        'Фурманова',
        'Цурюпинская',
        'Чапаева',
        'Чекистов',
        'Чеслава Белинского',
        'Чудновского',
        'Шаумяна',
        'Щербакова',
        'Щорса',
        'Юрия Коцюбинского',
        'Якира',
    ]

    def city_prefix(self):
        return self.random_element(self.city_prefixes)

    def postcode(self):
        """The code consists of five digits (01000-99999)"""
        return f'{self.generator.random.randint(0, 10)}{self.generator.random.randint(1000, 10000)}'

    def street_prefix(self):
        return self.random_element(self.street_prefixes)

    def street_title(self):
        return self.random_element(self.street_titles)



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``ar_JO`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Jordan
    """

    license_formats = (
        '{{initials}}-####',
        '{{initials}}-#####',
    )

    def initials(self):
        """Generate an initial number for license plates."""
        return self.random_element([
            '1',  # Ministers
            '2', '3',  # Parliament
            '5',  # General Government

            '6',  # Aqaba free zone
            '7', '8',  # Diplomatic
            '9',  # Temporary
            '10', '23',  # Passenger cars
            '38', '39',  # Crew cabs
            '41', '42',  # Light goods vehicles
            '44',  # Tractors
            '46',  # Motorcycles and scooters
            '50',  # Taxi
            '56',  # Small buses
            '58',  # Coaches
            '60',  # HGVs
            '70',  # Rental Cars
            '71',  # Trailer
            '90',  # Army
            '95',  # Ambulance
            '96',  # Gendarmerie
            '99',  # Police
        ])

    def license_plate(self):
        """Generate a license plate."""
        pattern = self.random_element(self.license_formats)
        return self.numerify(self.generator.parse(pattern))



from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):

    DAY_NAMES = {
        "0": "Կիրակի",
        "1": "Երկուշաբթի",
        "2": "Երեքշաբթի",
        "3": "Չորեքշաբթի",
        "4": "Հինգշաբթի",
        "5": "Ուրբաթ",
        "6": "Շաբաթ",
    }

    MONTH_NAMES = {
        "01": "Հունվար",
        "02": "Փետրվար",
        "03": "Մարտ",
        "04": "Ապրիլ",
        "05": "Մայիս",
        "06": "Հունիս",
        "07": "Հուլիս",
        "08": "Օգոստոս",
        "09": "Սեպտեմբեր",
        "10": "Հոկտեմբեր",
        "11": "Նոյեմբեր",
        "12": "Դեկտեմբեր",
    }

    def day_of_week(self):
        day = self.date('%w')
        return self.DAY_NAMES[day]

    def month_name(self):
        month = self.month()
        return self.MONTH_NAMES[month]



from .. import BaseProvider

localized = True

# 'Latin' is the default locale
default_locale = 'la'


class Provider(BaseProvider):
    """Implement default lorem provider for Faker.

    .. important::
       The default locale of the lorem provider is ``la``. When using a locale
       without a localized lorem provider, the ``la`` lorem provider will be
       used, so generated words will be in pseudo-Latin. The locale used for
       the standard provider docs was ``en_US``, and ``en_US`` has a localized
       lorem provider which is why the samples here show words in American
       English.
    """

    word_connector = ' '
    sentence_punctuation = '.'

    def words(self, nb=3, ext_word_list=None, unique=False):
        """Generate a tuple of words.

        The ``nb`` argument controls the number of words in the resulting list,
        and if ``ext_word_list`` is provided, words from that list will be used
        instead of those from the locale provider's built-in word list.

        If ``unique`` is ``True``, this method will return a list containing
        unique words. Under the hood, |random_sample| will be used for sampling
        without replacement. If ``unique`` is ``False``, |random_choices| is
        used instead, and the list returned may contain duplicates.

        .. warning::
           Depending on the length of a locale provider's built-in word list or
           on the length of ``ext_word_list`` if provided, a large ``nb`` can
           exhaust said lists if ``unique`` is ``True``, raising an exception.

        :sample:
        :sample: nb=5
        :sample: nb=5, ext_word_list=['abc', 'def', 'ghi', 'jkl']
        :sample: nb=4, ext_word_list=['abc', 'def', 'ghi', 'jkl'], unique=True
        """
        word_list = ext_word_list if ext_word_list else self.word_list
        if unique:
            return self.random_sample(word_list, length=nb)
        return self.random_choices(word_list, length=nb)

    def word(self, ext_word_list=None):
        """Generate a word.

        This method uses |words| under the hood with the ``nb`` argument set to
        ``1`` to generate the result.

        :sample:
        :sample: ext_word_list=['abc', 'def', 'ghi', 'jkl']
        """
        return self.words(1, ext_word_list)[0]

    def sentence(self, nb_words=6, variable_nb_words=True, ext_word_list=None):
        """Generate a sentence.

        The ``nb_words`` argument controls how many words the sentence will
        contain, and setting ``variable_nb_words`` to ``False`` will generate
        the exact amount, while setting it to ``True`` (default) will generate
        a random amount (+/-40%, minimum of 1) using |randomize_nb_elements|.

        Under the hood, |words| is used to generate the words, so the argument
        ``ext_word_list`` works in the same way here as it would in that method.

        :sample: nb_words=10
        :sample: nb_words=10, variable_nb_words=False
        :sample: nb_words=10, ext_word_list=['abc', 'def', 'ghi', 'jkl']
        :sample: nb_words=10, variable_nb_words=True,
                 ext_word_list=['abc', 'def', 'ghi', 'jkl']
        """
        if nb_words <= 0:
            return ''

        if variable_nb_words:
            nb_words = self.randomize_nb_elements(nb_words, min=1)

        words = list(self.words(nb=nb_words, ext_word_list=ext_word_list))
        words[0] = words[0].title()

        return self.word_connector.join(words) + self.sentence_punctuation

    def sentences(self, nb=3, ext_word_list=None):
        """Generate a list of sentences.

        This method uses |sentence| under the hood to generate sentences, and
        the ``nb`` argument controls exactly how many sentences the list will
        contain. The ``ext_word_list`` argument works in exactly the same way
        as well.

        :sample:
        :sample: nb=5
        :sample: nb=5, ext_word_list=['abc', 'def', 'ghi', 'jkl']
        """
        return [self.sentence(ext_word_list=ext_word_list)
                for _ in range(0, nb)]

    def paragraph(
            self,
            nb_sentences=3,
            variable_nb_sentences=True,
            ext_word_list=None):
        """Generate a paragraph.

        The ``nb_sentences`` argument controls how many sentences the paragraph
        will contain, and setting ``variable_nb_sentences`` to ``False`` will
        generate the exact amount, while setting it to ``True`` (default) will
        generate a random amount (+/-40%, minimum of 1) using
        |randomize_nb_elements|.

        Under the hood, |sentences| is used to generate the sentences, so the
        argument ``ext_word_list`` works in the same way here as it would in
        that method.

        :sample: nb_sentences=5
        :sample: nb_sentences=5, variable_nb_sentences=False
        :sample: nb_sentences=5, ext_word_list=['abc', 'def', 'ghi', 'jkl']
        :sample: nb_sentences=5, variable_nb_sentences=False,
                 ext_word_list=['abc', 'def', 'ghi', 'jkl']
        """
        if nb_sentences <= 0:
            return ''

        if variable_nb_sentences:
            nb_sentences = self.randomize_nb_elements(nb_sentences, min=1)

        para = self.word_connector.join(self.sentences(
            nb_sentences, ext_word_list=ext_word_list,
        ))

        return para

    def paragraphs(self, nb=3, ext_word_list=None):
        """Generate a list of paragraphs.

        This method uses |paragraph| under the hood to generate paragraphs, and
        the ``nb`` argument controls exactly how many sentences the list will
        contain. The ``ext_word_list`` argument works in exactly the same way
        as well.

        :sample: nb=5
        :sample: nb=5, ext_word_list=['abc', 'def', 'ghi', 'jkl']
        """
        return [self.paragraph(ext_word_list=ext_word_list)
                for _ in range(0, nb)]

    def text(self, max_nb_chars=200, ext_word_list=None):
        """Generate a text string.

        The ``max_nb_chars`` argument controls the approximate number of
        characters the text string will have, and depending on its value, this
        method may use either |words|, |sentences|, or |paragraphs| for text
        generation. The ``ext_word_list`` argument works in exactly the same way
        it would in any of those methods.

        :sample: max_nb_chars=20
        :sample: max_nb_chars=80
        :sample: max_nb_chars=160
        :sample: ext_word_list=['abc', 'def', 'ghi', 'jkl']
        """
        text = []
        if max_nb_chars < 5:
            raise ValueError(
                'text() can only generate text of at least 5 characters')

        if max_nb_chars < 25:
            # join words
            while not text:
                size = 0
                # determine how many words are needed to reach the $max_nb_chars
                # once;
                while size < max_nb_chars:
                    word = (self.word_connector if size else '') + \
                        self.word(ext_word_list=ext_word_list)
                    text.append(word)
                    size += len(word)
                text.pop()
            text[0] = text[0][0].upper() + text[0][1:]
            last_index = len(text) - 1
            text[last_index] += self.sentence_punctuation
        elif max_nb_chars < 100:
            # join sentences
            while not text:
                size = 0
                # determine how many sentences are needed to reach the
                # $max_nb_chars once
                while size < max_nb_chars:
                    sentence = (self.word_connector if size else '') + \
                        self.sentence(ext_word_list=ext_word_list)
                    text.append(sentence)
                    size += len(sentence)
                text.pop()
        else:
            # join paragraphs
            while not text:
                size = 0
                # determine how many paragraphs are needed to reach the
                # $max_nb_chars once
                while size < max_nb_chars:
                    paragraph = ('\n' if size else '') + \
                        self.paragraph(ext_word_list=ext_word_list)
                    text.append(paragraph)
                    size += len(paragraph)
                text.pop()

        return "".join(text)

    def texts(self, nb_texts=3, max_nb_chars=200, ext_word_list=None):
        """Generate a list of text strings.

        The ``nb_texts`` argument controls how many text strings the list will
        contain, and this method uses |text| under the hood for text generation,
        so the two remaining arguments, ``max_nb_chars`` and ``ext_word_list``
        will work in exactly the same way as well.

        :sample: nb_texts=5
        :sample: nb_texts=5, max_nb_chars=50
        :sample: nb_texts=5, max_nb_chars=50,
                 ext_word_list=['abc', 'def', 'ghi', 'jkl']
        """
        return [self.text(max_nb_chars, ext_word_list)
                for _ in range(0, nb_texts)]



from collections import OrderedDict

from .. import Provider as ColorProvider

localized = True


class Provider(ColorProvider):
    """Implement color provider for ``es_ES`` locale."""

    all_colors = OrderedDict((
        ("Agua marina medio", "#66CDAA"),
        ("Agua-marina", "#7FFFD4"),
        ("Almendra blanqueado", "#FFEBCD"),
        ("Amarillo", "#FFFF00"),
        ("Amarillo claro", "#FFFFE0"),
        ("Amarillo dorado", "#DAA520"),
        ("Amarillo dorado claro", "#FAFAD2"),
        ("Amarillo dorado oscuro", "#B8860B"),
        ("Amarillo dorado pálido", "#EEE8AA"),
        ("Amarillo trigo", "#F5DEB3"),
        ("Amarillo verde", "#9ACD32"),
        ("Azul", "#0000FF"),
        ("Azul Alicia", "#F0F8FF"),
        ("Azul acero", "#4682B4"),
        ("Azul acero claro", "#B0C4DE"),
        ("Azul anciano", "#6495ED"),
        ("Azul azur", "#F0FFFF"),
        ("Azul cadete", "#5F9EA0"),
        ("Azul cielo", "#87CEEB"),
        ("Azul cielo claro", "#87CEFA"),
        ("Azul cielo profundo", "#00BFFF"),
        ("Azul claro", "#ADD8E6"),
        ("Azul lona", "#1E90FF"),
        ("Azul marino", "#000080"),
        ("Azul medianoche", "#191970"),
        ("Azul medio", "#0000CD"),
        ("Azul oscuro", "#00008B"),
        ("Azul pizarra", "#6A5ACD"),
        ("Azul pizarra medio", "#7B68EE"),
        ("Azul pizarra oscuro", "#483D8B"),
        ("Azul polvo", "#B0E0E6"),
        ("Azul real", "#4169E1"),
        ("Azul violeta", "#8A2BE2"),
        ("Beige", "#F5F5DC"),
        ("Beige antiguo", "#FAEBD7"),
        ("Beige limón", "#FFFACD"),
        ("Beige melocotón", "#FFDAB9"),
        ("Beige mocasín", "#FFE4B5"),
        ("Beige papaya", "#FFEFD5"),
        ("Bisque", "#FFE4C4"),
        ("Blanco", "#FFFFFF"),
        ("Blanco concha", "#FFF5EE"),
        ("Blanco encaje", "#FDF5E6"),
        ("Blanco fantasma", "#F8F8FF"),
        ("Blanco floral", "#FFFAF0"),
        ("Blanco humo", "#F5F5F5"),
        ("Blanco lavanda", "#FFF0F5"),
        ("Blanco lino", "#FAF0E6"),
        ("Blanco menta", "#F5FFFA"),
        ("Blanco navajo", "#FFDEAD"),
        ("Blanco nieve", "#FFFAFA"),
        ("Caqui", "#6B8E23"),
        ("Caqui oscuro", "#BDB76B"),
        ("Chartreuse", "#7FFF00"),
        ("Chocolate", "#D2691E"),
        ("Cian", "#00FFFF"),
        ("Cian clarto", "#E0FFFF"),
        ("Ciruela", "#DDA0DD"),
        ("Coral", "#FF7F50"),
        ("Coral claro", "#F08080"),
        ("Amarillo maíz dulce", "#FFF8DC"),
        ("Cyan oscuro", "#008B8B"),
        ("Fucsia", "#FF00FF"),
        ("Granate", "#800000"),
        ("Gris", "#808080"),
        ("Gris claro", "#D3D3D3"),
        ("Gris gainsboro (Estaño)", "#DCDCDC"),
        ("Gris mate", "#696969"),
        ("Gris oscuro", "#A9A9A9"),
        ("Gris pizarra", "#708090"),
        ("Gris pizarra claro", "#778899"),
        ("Gris pizarra oscuro", "#2F4F4F"),
        ("Lavanda", "#E6E6FA"),
        ("Lima", "#00FF00"),
        ("Magenta", "#FF00FF"),
        ("Magenta oscuro", "#8B008B"),
        ("Marfil", "#FFFFF0"),
        ("Marrón", "#A52A2A"),
        ("Marrón arena", "#F4A460"),
        ("Marrón caqui", "#F0E68C"),
        ("Marrón cuero", "#8B4513"),
        ("Marrón madera rústica", "#DEB887"),
        ("Marrón perú", "#CD853F"),
        ("Marrón rojizo", "#D2B48C"),
        ("Marrón rosado", "#BC8F8F"),
        ("Marrón siena", "#A0522D"),
        ("Melón dulce", "#F0FFF0"),
        ("Naranja", "#FFA500"),
        ("Naranja oscuro", "#FF8C00"),
        ("Negro", "#000000"),
        ("Oliva", "#808000"),
        ("Oro", "#FFD700"),
        ("Orquídea", "#DA70D6"),
        ("Orquídea medio", "#BA55D3"),
        ("Orquídea púrpura oscuro", "#9932CC"),
        ("Plata", "#C0C0C0"),
        ("Púrpura", "#800080"),
        ("Púrpura medio", "#9370DB"),
        ("Rojo", "#FF0000"),
        ("Rojo anaranjado", "#FF4500"),
        ("Rojo carmesí", "#DC143C"),
        ("Rojo indio", "#CD5C5C"),
        ("Rojo ladrillo", "#B22222"),
        ("Rojo oscuro", "#8B0000"),
        ("Rojo tomate", "#FF6347"),
        ("Rojo violeta medio", "#C71585"),
        ("Rosa", "#FFC0CB"),
        ("Rosa brumoso", "#FFE4E1"),
        ("Rosa caliente", "#FF69B4"),
        ("Rosa claro", "#FFB6C1"),
        ("Rosa profundo", "#FF1493"),
        ("Salmón", "#FA8072"),
        ("Salmón claro", "#FFA07A"),
        ("Salmón oscuro", "#E9967A"),
        ("Turquesa", "#40E0D0"),
        ("Turquesa medio", "#48D1CC"),
        ("Turquesa oscuro", "#00CED1"),
        ("Turquesa pálido", "#AFEEEE"),
        ("Verde", "#008000"),
        ("Verde azulado", "#008080"),
        ("Verde bosque", "#228B22"),
        ("Verde claro", "#90EE90"),
        ("Verde lima", "#32CD32"),
        ("Verde limón", "#ADFF2F"),
        ("Verde mar", "#2E8B57"),
        ("Verde mar claro", "#20B2AA"),
        ("Verde mar medio", "#3CB371"),
        ("Verde mar oscuro", "#8FBC8F"),
        ("Verde oliva oscuro", "#556B2F"),
        ("Verde oscuro", "#006400"),
        ("Verde prado", "#7CFC00"),
        ("Verde primavera", "#00FF7F"),
        ("Verde primavera medio", "#00FA9A"),
        ("Verde pálido", "#98FB98"),
        ("Violeta", "#EE82EE"),
        ("Violeta cardo", "#D8BFD8"),
        ("Violeta oscuro", "#9400D3"),
        ("Violeta sonrojado pálido", "#DB7093"),
        ("Índigo", "#4B0082"),
    ))

    safe_colors = (
        "negro", "budeos", "verde", "rojo",
        "violeta", "verde azulado", "azul", "plata",
        "gris", "amarilo", "fucsia", "cian", "blanco",
    )



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats_female = (
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}-{{last_name}}',
        '{{first_name_female}}-{{first_name_female}} {{last_name}}',
    )

    formats_male = (
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}-{{last_name}}',
        '{{first_name_male}}-{{first_name_male}} {{last_name}}',
    )

    formats = formats_male + formats_female

    # Source:
    # https://www.retraitequebec.gouv.qc.ca/fr/services-en-ligne-outils/banque-de-prenoms/Pages/banque-de-prenoms.aspx
    first_names_male = (
        'Alain',
        'Alexandre',
        'Alexis',
        'André',
        'Antoine',
        'Arthur',
        'Benjamin',
        'Benoît',
        'Bernard',
        'Bertrand',
        'Charles',
        'Daniel',
        'David',
        'Denis',
        'Édouard',
        'Émile',
        'Emmanuel',
        'Éric',
        'Étienne',
        'François',
        'Frédéric',
        'Gabriel',
        'Georges',
        'Gérard',
        'Gilbert',
        'Gilles',
        'Grégoire',
        'Guillaume',
        'Guy',
        'William',
        'Henri',
        'Hugues',
        'Jacques',
        'Jean',
        'Jérôme',
        'Jonathan',
        'Joseph',
        'Jules',
        'Julien',
        'Kevin',
        'Laurent',
        'Louis',
        'Luc',
        'Lucas',
        'Marc',
        'Maxime',
        'Marcel',
        'Martin',
        'Mathieu',
        'Maurice',
        'Michel',
        'Nathan',
        'Nicolas',
        'Noël',
        'Olivier',
        'Patrick',
        'Paul',
        'Philippe',
        'Pierre',
        'Raphaël',
        'Raymond',
        'Rémy',
        'René',
        'Richard',
        'Robert',
        'Roger',
        'Roland',
        'Samuel',
        'Sébastien',
        'Stéphane',
        'Théodore',
        'Thomas',
        'Timothée',
        'Tristan',
        'Victor',
        'Vincent',
        'Xavier',
        'Yves',
        'Zacharie',
    )

    first_names_female = (
        'Agnès',
        'Alexandra',
        'Alex',
        'Alice',
        'Amélie',
        'Anaïs',
        'Andrée',
        'Anne',
        'Anouk',
        'Astrid',
        'Audrey',
        'Aurélie',
        'Aurore',
        'Béatrice',
        'Brigitte',
        'Camille',
        'Caroline',
        'Catherine',
        'Cécile',
        'Céline',
        'Célina',
        'Chantal',
        'Charlotte',
        'Christelle',
        'Christiane',
        'Christine',
        'Claire',
        'Claudine',
        'Claude',
        'Clémence',
        'Colette',
        'Constance',
        'Danielle',
        'Denise',
        'Diane',
        'Dominique',
        'Dorothée',
        'Édith',
        'Emma',
        'Éléonore',
        'Élisabeth',
        'Élise',
        'Élodie',
        'Émilie',
        'Emmanuelle',
        'Florence',
        'Françoise',
        'Frédérique',
        'Gabrielle',
        'Geneviève',
        'Hélène',
        'Henriette',
        'Hortense',
        'Isabelle',
        'Jacqueline',
        'Jeanne',
        'Jeannine',
        'Jessica',
        'Joséphine',
        'Josette',
        'Julie',
        'Juliette',
        'Karine',
        'Laetitia',
        'Laurence',
        'Laurie',
        'Lorraine',
        'Louise',
        'Lucie',
        'Lucy',
        'Manon',
        'Marcelle',
        'Marguerite',
        'Margot',
        'Margaret',
        'Marianne',
        'Marie',
        'Martine',
        'Maryse',
        'Maude',
        'Mathilde',
        'Mélanie',
        'Michèle',
        'Michelle',
        'Monique',
        'Nathalie',
        'Nathalie',
        'Nicole',
        'Noémie',
        'Océane',
        'Odette',
        'Olivia',
        'Patricia',
        'Paulette',
        'Pauline',
        'Pénélope',
        'Renée',
        'Rosalie',
        'Sarah',
        'Simone',
        'Sophie',
        'Stéphanie',
        'Susanne',
        'Sylvie',
        'Thérèse',
        'Vanessa',
        'Valérie',
        'Véronique',
        'Virginie',
        'Zoé',
    )

    first_names = first_names_male + first_names_female

    # Source:
    # https://fr.wikipedia.org/wiki/Liste_des_noms_de_famille_les_plus_courants_au_Qu%C3%A9bec
    last_names = (
        'Tremblay',
        'Gagnon',
        'Roy',
        'Côté',
        'Bouchard',
        'Gauthier',
        'Morin',
        'Lavoie',
        'Fortin',
        'Gagné',
        'Ouellet',
        'Pelletier',
        'Bélanger',
        'Lévesque',
        'Bergeron',
        'Leblanc',
        'Paquette',
        'Girard',
        'Simard',
        'Boucher',
        'Caron',
        'Beaulieu',
        'Cloutier',
        'Dubé',
        'Poirier',
        'Fournier',
        'Lapointe',
        'Leclerc',
        'Lefebvre',
        'Poulin',
        'Thibault',
        'St-Pierre',
        'Nadeau',
        'Martin',
        'Landry',
        'Martel',
        'Bédard',
        'Grenier',
        'Lessard',
        'Bernier',
        'Richard',
        'Michaud',
        'Hébert',
        'Desjardins',
        'Couture',
        'Turcotte',
        'Lachance',
        'Parent',
        'Blais',
        'Gosselin',
        'Savard',
        'Proulx',
        'Beaudoin',
        'Demers',
        'Perreault',
        'Boudreau',
        'Lemieux',
        'Cyr',
        'Perron',
        'Dufour',
        'Dion',
        'Mercier',
        'Bolduc',
        'Bérubé',
        'Boisvert',
        'Langlois',
        'Ménard',
        'Therrien',
        'Plante',
        'Bilodeau',
        'Blanchette',
        'Dubois',
        'Champagne',
        'Paradis',
        'Fortier',
        'Arsenault',
        'Dupuis',
        'Gaudreault',
        'Hamel',
        'Houle',
        'Villeneuve',
        'Rousseau',
        'Gravel',
        'Thériault',
        'Lemay',
        'Robert',
        'Allard',
        'Deschênes',
        'Giroux',
        'Guay',
        'Leduc',
        'Boivin',
        'Charbonneau',
        'Lambert',
        'Raymond',
        'Vachon',
        'Gilbert',
        'Audet',
        'Jean',
        'Larouche',
        'Legault',
        'Trudel',
        'Fontaine',
        'Picard',
        'Labelle',
        'Lacroix',
        'Jacques',
        'Moreau',
        'Carrier',
        'Bernard',
        'Desrosiers',
        'Goulet',
        'Renaud',
        'Dionne',
        'Lapierre',
        'Vaillancourt',
        'Fillion',
        'Lalonde',
        'Tessier',
        'Bertrand',
        'Tardif',
        'Lepage',
        'Gingras',
        'Benoît',
        'Rioux',
        'Giguère',
        'Drouin',
        'Harvey',
        'Lauzon',
        'Nguyen',
        'Gendron',
        'Boutin',
        'Laflamme',
        'Vallée',
        'Dumont',
        'Breton',
        'Paré',
        'Paquin',
        'Robitaille',
        'Gélinas',
        'Duchesne',
        'Lussier',
        'Séguin',
        'Veilleux',
        'Potvin',
        'Gervais',
        'Pépin',
        'Laroche',
        'Morissette',
        'Charron',
        'Lavallée',
        'Laplante',
        'Chabot',
        'Brunet',
        'Vézina',
        'Desrochers',
        'Labrecque',
        'Coulombe',
        'Tanguay',
        'Chouinard',
        'Noël',
        'Pouliot',
        'Lacasse',
        'Daigle',
        'Marcoux',
        'Lamontagne',
        'Turgeon',
        'Larocque',
        'Roberge',
        'Auger',
        'Massé',
        'Pilon',
        'Racine',
        'Dallaire',
        'Émond',
        'Grégoire',
        'Beauregard',
        'Smith',
        'Denis',
        'Lebel',
        'Blouin',
        'Martineau',
        'Labbé',
        'Beauchamp',
        'St-Onge',
        'Charette',
        'Dupont',
        'Létourneau',
        'Rodrigue',
        'Cormier',
        'Rivard',
        'Mathieu',
        'Asselin',
        'St-Jean',
        'Plourde',
        'Thibodeau',
        'Bélisle',
        'St-Laurent',
        'Godin',
        'Desbiens',
        'Lavigne',
        'Doucet',
        'Labonté',
        'Marchand',
        'Brassard',
        'Forget',
        'Patel',
        'Marcotte',
        'Béland',
        'Larose',
        'Duval',
        'Archambault',
        'Maltais',
        'Trépanier',
        'Laliberté',
        'Bisson',
        'Brisson',
        'Dufresne',
        'Beaudry',
        'Chartrand',
        'Houde',
        'Fréchette',
        'Lafontaine',
        'Guillemette',
        'Drolet',
        'Vincent',
        'Richer',
        'Germain',
        'Larivière',
        'Ferland',
        'Trottier',
        'Piché',
        'Boulanger',
        'Sirois',
        'Charest',
        'Provost',
        'Durand',
        'Dumas',
        'Soucy',
        'Lamoureux',
        'Lachapelle',
        'Bégin',
        'Boily',
        'Croteau',
        'Savoie',
        'Provencher',
        'Prévost',
        'Duguay',
        'Lemire',
        'Delisle',
    )



"""
SSN provider for es_MX.

This module adds a provider for mexican SSN, along with Unique Population
Registry Code (CURP) and Federal Taxpayer Registry ID (RFC).
"""

import random
import string

from .. import Provider as BaseProvider

ALPHABET = string.ascii_uppercase
ALPHANUMERIC = string.digits + ALPHABET
VOWELS = "AEIOU"
CONSONANTS = [
    letter
    for letter in ALPHABET
    if letter not in VOWELS
]

# https://es.wikipedia.org/wiki/Plantilla:Abreviaciones_de_los_estados_de_M%C3%A9xico
STATES_RENAPO = [
    "AS",
    "BC",
    "BS",
    "CC",
    "CS",
    "CH",
    "DF",
    "CL",
    "CM",
    "DG",
    "GT",
    "GR",
    "HG",
    "JC",
    "MC",
    "MN",
    "MS",
    "NT",
    "NL",
    "OC",
    "PL",
    "QO",
    "QR",
    "SP",
    "SL",
    "SR",
    "TC",
    "TS",
    "TL",
    "VZ",
    "YN",
    "ZS",
    "NE",  # Foreign Born
]

FORBIDDEN_WORDS = {
    "BUEI": "BUEX",
    "BUEY": "BUEX",
    "CACA": "CACX",
    "CACO": "CACX",
    "CAGA": "CAGX",
    "CAGO": "CAGX",
    "CAKA": "CAKX",
    "CAKO": "CAKX",
    "COGE": "COGX",
    "COJA": "COJX",
    "COJE": "COJX",
    "COJI": "COJX",
    "COJO": "COJX",
    "CULO": "CULX",
    "FETO": "FETX",
    "GUEY": "GUEX",
    "JOTO": "JOTX",
    "KACA": "KACX",
    "KACO": "KACX",
    "KAGA": "KAGX",
    "KAGO": "KAGX",
    "KOGE": "KOGX",
    "KOJO": "KOJX",
    "KAKA": "KAKX",
    "KULO": "KULX",
    "MAME": "MAMX",
    "MAMO": "MAMX",
    "MEAR": "MEAX",
    "MEAS": "MEAX",
    "MEON": "MEOX",
    "MION": "MIOX",
    "MOCO": "MOCX",
    "MULA": "MULX",
    "PEDA": "PEDX",
    "PEDO": "PEDX",
    "PENE": "PENX",
    "PUTA": "PUTX",
    "PUTO": "PUTX",
    "QULO": "QULX",
    "RATA": "RATX",
    "RUIN": "RUIN",
}

CURP_CHARACTERS = "0123456789ABCDEFGHIJKLMNÑOPQRSTUVWXYZ"


def _reduce_digits(number):
    """
    Sum of digits of a number until sum becomes single digit.

    Example:
        658 => 6 + 5 + 8 = 19 => 1 + 9 = 10 => 1
    """
    if number == 0:
        return 0
    if number % 9 == 0:
        return 9

    return number % 9


def ssn_checksum(digits):
    """
    Calculate the checksum for the mexican SSN (IMSS).
    """
    return -sum(
        _reduce_digits(n * (i % 2 + 1))
        for i, n in enumerate(digits)
    ) % 10


def curp_checksum(characters):
    """
    Calculate the checksum for the mexican CURP.
    """
    start = 18
    return -sum(
        (start - i) * CURP_CHARACTERS.index(n)
        for i, n in enumerate(characters)
    ) % 10


class Provider(BaseProvider):
    """
    A Faker provider for the Mexican SSN, RFC and CURP
    """
    ssn_formats = ("###########",)

    def ssn(self):
        """
        Mexican Social Security Number, as given by IMSS.

        :return: a random Mexican SSN
        """
        office = self.random_int(min=1, max=99)
        birth_year = self.random_int(min=0, max=99)
        start_year = self.random_int(min=0, max=99)
        serial = self.random_int(min=1, max=9999)

        num = f'{office:02d}{start_year:02d}{birth_year:02d}{serial:04d}'

        check = ssn_checksum(map(int, num))
        num += str(check)

        return num

    def curp(self):
        """
        See https://es.wikipedia.org/wiki/Clave_%C3%9Anica_de_Registro_de_Poblaci%C3%B3n.

        :return: a random Mexican CURP (Unique Population Registry Code)
        """
        birthday = self.generator.date_of_birth()

        first_surname = random.choice(ALPHABET) + random.choice(VOWELS)
        second_surname = random.choice(ALPHABET)
        given_name = random.choice(ALPHABET)
        name_initials = first_surname + second_surname + given_name

        birth_date = birthday.strftime("%y%m%d")
        gender = random.choice("HM")
        state = random.choice(STATES_RENAPO)
        first_surname_inside = random.choice(CONSONANTS)
        second_surname_inside = random.choice(CONSONANTS)
        given_name_inside = random.choice(ALPHABET)

        # This character is assigned to avoid duplicity
        # It's normally '0' for those born < 2000
        # and 'A' for those born >= 2000
        assigned_character = "0" if birthday.year < 2000 else "A"

        name_initials = FORBIDDEN_WORDS.get(name_initials, name_initials)

        random_curp = (
            name_initials +
            birth_date +
            gender +
            state +
            first_surname_inside +
            second_surname_inside +
            given_name_inside +
            assigned_character
        )

        random_curp += str(curp_checksum(random_curp))

        return random_curp

    def rfc(self, natural=True):
        """
        See https://es.wikipedia.org/wiki/Registro_Federal_de_Contribuyentes

        :param natural: Whether to return the RFC of a natural person.
            Otherwise return the RFC of a legal person.
        :type natural: bool
        :return: a random Mexican RFC
        """
        birthday = self.generator.date_of_birth()

        if natural:
            first_surname = random.choice(ALPHABET) + random.choice(VOWELS)
            second_surname = random.choice(ALPHABET)
            given_name = random.choice(ALPHABET)
            name_initials = first_surname + second_surname + given_name
        else:
            name_initials = (
                self.random_uppercase_letter() +
                self.random_uppercase_letter() +
                self.random_uppercase_letter()
            )

        birth_date = birthday.strftime("%y%m%d")
        disambiguation_code = (
            random.choice(ALPHANUMERIC) +
            random.choice(ALPHANUMERIC) +
            random.choice(ALPHANUMERIC)
        )

        random_rfc = (
            name_initials +
            birth_date +
            disambiguation_code
        )

        return random_rfc



from collections import OrderedDict
from operator import itemgetter

from .. import Provider as PersonProvider


class Provider(PersonProvider):
    # link: http://dic.nicovideo.jp/a/日本人の名前一覧
    # link: http://www.meijiyasuda.co.jp/enjoy/ranking/
    first_name_female_pairs = (
        ('明美', 'アケミ', 'Akemi'),
        ('あすか', 'アスカ', 'Asuka'),
        ('香織', 'カオリ', 'Kaori'),
        ('加奈', 'カナ', 'Kana'),
        ('くみ子', 'クミコ', 'Kumiko'),
        ('さゆり', 'サユリ', 'Sayuri'),
        ('知実', 'サトミ', 'Satomi'),
        ('千代', 'チヨ', 'Chiyo'),
        ('直子', 'ナオコ', 'Naoko'),
        ('七夏', 'ナナミ', 'Nanami'),
        ('花子', 'ハナコ', 'Hanako'),
        ('春香', 'ハルカ', 'Haruka'),
        ('真綾', 'マアヤ', 'Maaya'),
        ('舞', 'マイ', 'Mai'),
        ('美加子', 'ミカコ', 'Mikako'),
        ('幹', 'ミキ', 'Miki'),
        ('桃子', 'モモコ', 'Momoko'),
        ('結衣', 'ユイ', 'Yui'),
        ('裕美子', 'ユミコ', 'Yumiko'),
        ('陽子', 'ヨウコ', 'Yoko'),
        ('里佳', 'リカ', 'Rika'),
    )

    # for backwards compatibility
    first_names_female = tuple(map(itemgetter(0), first_name_female_pairs))
    first_kana_names_female = tuple(map(itemgetter(1), first_name_female_pairs))
    first_romanized_names_female = tuple(map(itemgetter(2), first_name_female_pairs))

    first_name_male_pairs = (
        ('晃', 'アキラ', 'Akira'),
        ('篤司', 'アツシ', 'Atsushi'),
        ('治', 'オサム', 'Osamu'),
        ('和也', 'カズヤ', 'Kazuya'),
        ('京助', 'キョウスケ', 'Kyosuke'),
        ('健一', 'ケンイチ', 'Kenichi'),
        ('修平', 'シュウヘイ', 'Shohei'),
        ('翔太', 'ショウタ', 'Shota'),
        ('淳', 'ジュン', 'Jun'),
        ('聡太郎', 'ソウタロウ', 'Sotaro'),
        ('太一', 'タイチ', 'Taichi'),
        ('太郎', 'タロウ', 'Taro'),
        ('拓真', 'タクマ', 'Takuma'),
        ('翼', 'ツバサ', 'Tsubasa'),
        ('智也', 'トモヤ', 'Tomoya'),
        ('直樹', 'ナオキ', 'Naoki'),
        ('直人', 'ナオト', 'Naoto'),
        ('英樹', 'ヒデキ', 'Hideki'),
        ('浩', 'ヒロシ', 'Hiroshi'),
        ('学', 'マナブ', 'Manabu'),
        ('充', 'ミツル', 'Mituru'),
        ('稔', 'ミノル', 'Minoru'),
        ('裕樹', 'ユウキ', 'Yuki'),
        ('裕太', 'ユウタ', 'Yuta'),
        ('康弘', 'ヤスヒロ', 'Yasuhiro'),
        ('陽一', 'ヨウイチ', 'Yoichi'),
        ('洋介', 'ヨウスケ', 'Yosuke'),
        ('亮介', 'リョウスケ', 'Ryosuke'),
        ('涼平', 'リョウヘイ', 'Ryohei'),
        ('零', 'レイ', 'Rei'),
    )

    # for backwards compatibility
    first_names_male = tuple(map(itemgetter(0), first_name_male_pairs))
    first_kana_names_male = tuple(map(itemgetter(1), first_name_male_pairs))
    first_romanized_names_male = tuple(map(itemgetter(2), first_name_male_pairs))

    # for backwards compatibility
    first_names = first_names_male + first_names_female
    first_kana_names = first_kana_names_male + first_kana_names_female
    first_romanized_names = first_romanized_names_male \
        + first_romanized_names_female

    first_name_pairs = first_name_male_pairs + first_name_female_pairs

    last_name_pairs = OrderedDict((
        (("佐藤", "サトウ", "Sato"), 366803),
        (("鈴木", "スズキ", "Suzuki"), 321135),
        (("高橋", "タカハシ", "Takahashi"), 266782),
        (("田中", "タナカ", "Tanaka"), 245821),
        (("伊藤", "イトウ", "Ito"), 203357),
        (("渡辺", "ワタナベ", "Watanabe"), 200504),
        (("山本", "ヤマモト", "Yamamoto"), 200134),
        (("中村", "ナカムラ", "Nakamura"), 195219),
        (("小林", "コバヤシ", "Kobayashi"), 191819),
        (("加藤", "カトウ", "Kato"), 160283),
        (("吉田", "ヨシダ", "Yoshida"), 154461),
        (("山田", "ヤマダ", "Yamada"), 151675),
        (("佐々木", "ササキ", "Sasaki"), 135927),
        (("山口", "ヤマグチ", "Yamaguchi"), 119501),
        (("松本", "マツモト", "Matsumoto"), 116490),
        (("井上", "イノウエ", "Inoue"), 111287),
        (("木村", "キムラ", "Kimura"), 107446),
        (("林", "ハヤシ", "Hayashi"), 101826),
        (("斎藤", "サイトウ", "Saito"), 101774),
        (("清水", "シミズ", "Shimizu"), 97826),
        (("山崎", "ヤマザキ", "Yamazaki"), 90781),
        (("阿部", "アベ", "Abe"), 86833),
        (("森", "モリ", "Mori"), 86507),
        (("池田", "イケダ", "Ikeda"), 84860),
        (("橋本", "ハシモト", "Hashimoto"), 82836),
        (("山下", "ヤマシタ", "Yamashita"), 80588),
        (("石川", "イシカワ", "Ishikawa"), 77471),
        (("中島", "ナカジマ", "Nakajima"), 74106),
        (("前田", "マエダ", "Maeda"), 72930),
        (("藤田", "フジタ", "Fujita"), 72375),
        (("後藤", "ゴトウ", "Goto"), 71629),
        (("小川", "オガワ", "Ogawa"), 71179),
        (("岡田", "オカダ", "Okada"), 70347),
        (("長谷川", "ハセガワ", "Hasegawa"), 69201),
        (("村上", "ムラカミ", "Murakami"), 68606),
        (("近藤", "コンドウ", "Kondo"), 68297),
        (("石井", "イシイ", "Ishii"), 67079),
        (("遠藤", "エンドウ", "Endo"), 62620),
        (("斉藤", "サイトウ", "Saito"), 62540),
        (("坂本", "サカモト", "Sakamoto"), 62308),
        (("青木", "アオキ", "Aoki"), 59516),
        (("藤井", "フジイ", "Fujii"), 59204),
        (("西村", "ニシムラ", "Nishimura"), 58821),
        (("福田", "フクダ", "Fukuda"), 58714),
        (("太田", "オオタ", "Ota"), 58439),
        (("三浦", "ミウラ", "Miura"), 58006),
        (("藤原", "フジワラ", "Fujiwara"), 57742),
        (("松田", "マツダ", "Matsuda"), 55883),
        (("岡本", "オカモト", "Okamoto"), 55539),
        (("中川", "ナカガワ", "Nakagawa"), 55221),
    ))

    # for backwards compatibility only. use the pairs instead
    last_names = tuple(map(itemgetter(0), last_name_pairs))
    last_kana_names = tuple(map(itemgetter(1), last_name_pairs))
    last_romanized_names = tuple(map(itemgetter(2), last_name_pairs))

    formats_male = (
        '{{last_name}} {{first_name_male}}',
    )

    formats_female = (
        '{{last_name}} {{first_name_female}}',
    )

    formats = formats_male + formats_female

    kana_formats_male = (
        '{{last_kana_name}} {{first_kana_name_male}}',
    )

    kana_formats_female = (
        '{{last_kana_name}} {{first_kana_name_female}}',
    )

    kana_formats = kana_formats_male + kana_formats_female

    romanized_formats_male = (
        '{{first_romanized_name_male}} {{last_romanized_name}}',
    )

    romanized_formats_female = (
        '{{first_romanized_name_female}} {{last_romanized_name}}',
    )

    romanized_formats = romanized_formats_male + romanized_formats_female

    def first_name_pair(self):
        """
        @example ('明美', 'アケミ', 'Akemi')
        """
        return self.random_element(self.first_name_pairs)

    def first_name_male_pair(self):
        """
        @example ('晃', 'アキラ', 'Akira')
        """
        return self.random_element(self.first_name_male_pairs)

    def first_name_female_pair(self):
        """
        @example ('明美', 'アケミ', 'Akemi')
        """
        return self.random_element(self.first_name_female_pairs)

    def last_name_pair(self):
        """
        @example ('佐藤', 'サトウ', 'Sato')
        """
        return self.random_element(self.last_name_pairs)

    def first_name(self):
        """
        @example '明美'
        """
        return self.first_name_pair()[0]

    def first_name_male(self):
        """
        @example '晃'
        """
        return self.first_name_male_pair()[0]

    def first_name_female(self):
        """
        @example '明美'
        """
        return self.first_name_female_pair()[0]

    def last_name(self):
        """
        @example '佐藤'
        """
        return self.last_name_pair()[0]

    def first_kana_name(self):
        """
        @example 'アケミ'
        """
        return self.first_name_pair()[1]

    def first_kana_name_male(self):
        """
        @example 'アキラ'
        """
        return self.first_name_male_pair()[1]

    def first_kana_name_female(self):
        """
        @example 'アケミ'
        """
        return self.first_name_female_pair()[1]

    def last_kana_name(self):
        """
        @example 'サトウ'
        """
        return self.last_name_pair()[1]

    def first_romanized_name(self):
        """
        @example 'Akemi'
        """
        return self.first_name_pair()[2]

    def first_romanized_name_male(self):
        """
        @example 'Akira'
        """
        return self.first_name_male_pair()[2]

    def first_romanized_name_female(self):
        """
        @example 'Akemi'
        """
        return self.first_name_female_pair()[2]

    def last_romanized_name(self):
        """
        @example 'Sato'
        """
        return self.last_name_pair()[2]

    def kana_name(self):
        """
        @example 'サトウ アケミ'
        """
        pattern = self.random_element(self.kana_formats)
        return self.generator.parse(pattern)

    def kana_name_male(self):
        """
        @example 'サトウ アキラ'
        """
        pattern = self.random_element(self.kana_formats_male)
        return self.generator.parse(pattern)

    def kana_name_female(self):
        """
        @example 'サトウ アケミ'
        """
        pattern = self.random_element(self.kana_formats_female)
        return self.generator.parse(pattern)

    def romanized_name(self):
        """
        @example 'Akemi Sato'
        """
        pattern = self.random_element(self.romanized_formats)
        return self.generator.parse(pattern)

    def romanized_name_male(self):
        """
        @example 'Akira Sato'
        """
        pattern = self.random_element(self.romanized_formats_male)
        return self.generator.parse(pattern)

    def romanized_name_female(self):
        """
        @example 'Akemi Sato'
        """
        pattern = self.random_element(self.romanized_formats_female)
        return self.generator.parse(pattern)



from collections import OrderedDict

from ..en import Provider as AddressProvider


class Provider(AddressProvider):
    counties = (
        "Cork", "Galway", "Mayo", "Donegal", "Kerry", "Tipperary", "Clare", "Tyrone", "Antrim", "Limerick", "Roscommon",
        "Down", "Meath", "Londonderry", "Wexford", "Kilkenny", "Offaly", "Cavan", "Wicklow", "Waterford", "Sligo",
        "Laois", "Westmeath", "Kildare", "Leitrim", "Armagh", "Fermanagh", "Monaghan", "Dublin", "Louth", "Longford",
        "Carlow",
    )

    _postcode_sets = OrderedDict((
        (' ', [' ', '']),
        ('N', [str(i) for i in range(0, 10)]),
        ('L', 'ACDEFHKNPRTVWXY'),
        ('A', 'ACDEFHKNPRTVWXY0123456789'),
    ))
    postcode_pattern = "LNN AAAA"

    def postcode(self):
        postcode = ''
        for placeholder in self.postcode_pattern:
            postcode += self.random_element(self._postcode_sets[placeholder])
        return postcode

    def county(self):
        return self.random_element(self.counties)
